<?php
/**
* NOTICE OF LICENSE
*
* This file is licenced under the Software License Agreement.
* With the purchase or the installation of the software in your application
* you accept the licence agreement.
*
* You must not modify, adapt or create derivative works of this source code
*
*  @author    Sirv Limited <support@sirv.com>
*  @copyright Copyright (c) 2017-2023 Sirv Limited. All rights reserved
*  @license   https://www.magictoolbox.com/license/
*/
class SirvOptions extends Sirv
{
    public $sirv;
    public $l;

    public function __construct(Module $module, $translations)
    {
        $this->sirv = $module;
        $this->l = $translations;
    }

    public function getOptions()
    {
        if (defined('PS_ADMIN_DIR')) {
            $profileOptionValue = Configuration::get('SIRV_PROFILE');
            $profileName = (empty($profileOptionValue))?'Default':$profileOptionValue;
        } else {
            $psGeneratedImageSize = 0;
        }

        if (empty($profileName)) {
            $profileName = '';
        }
        if (empty($psGeneratedImageSize)) {
            $psGeneratedImageSize = '';
        }

        $link = new Link();

        $options = array(
            'active' => array(
                'id' => 'active',
                'group' => $this->l['Enable Sirv'],
                'order' => '1',
                'default' => '0',
                'label' => $this->l['Active'],
                'description' => '',
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),
            'domain' => array(
                'id' => 'active',
                'group' => $this->l['Enable Sirv'],
                'order' => '2',
                'default' => '',
                'label' => $this->l['Domain'],
                'description' =>
                    sprintf(
                        $this->l['Create and manage domains..'],
                        '<a href="https://my.sirv.com/#/account/settings/content-delivery/" target="_blank">',
                        '</a>'
                    ),
                'type' => 'array',
                'subType' => 'select',
                'values'  => array(),
                'scope' => 'module'
            ),
            'account_email' => array(
                'id'        => 'account_email',
                'hidden'    => true,
                'default'   => ''
            ),
           'stat' => array(
                'id'        => 'stat',
                'hidden'    => true,
                'default'   => ''
            ),
            'client_id' => array(
                'id'        => 'client_id',
                'hidden'    => true,
                'default'   => ''
            ),
            'client_secret' => array(
                'id'        => 'client_secret',
                'hidden'    => true,
                'default'   => ''
            ),
            'token' => array(
                'id'        => 'token',
                'hidden'    => true,
                'default'   => ''
            ),
            'token_expire_time' => array(
                'id'        => 'token_expire_time',
                'hidden'    => true,
                'default'   => ''
            ),
            'cdn_url' => array(
                'id'        => 'cdn_url',
                'hidden'    => true,
                'default'   => ''
            ),
            's3_bucket' => array(
                'hidden' => true,
                'id' => 's3_bucket',
                'group' => $this->l['Sirv account'],
                'group_description' => '',
                'order' => '10',
                'default'    => '',
                'label' => $this->l['Bucket'],
                'description'    => '',
                'type' => 'text',
                'scope' => 'module',
                'autocomplete' => 'off'
            ),
            's3_access_key' => array(
                'hidden' => true,
                'id' => 's3_access_key',
                'group' => $this->l['Sirv account'],
                'order' => '20',
                'default'    => '',
                'label' => $this->l['Access key'],
                'description'    => '',
                'type' => 'text',
                'scope' => 'module',
                'autocomplete' => 'off'
            ),
            's3_secret_key' => array(
                'hidden' => true,
                'id' => 's3_access_key',
                'group' => $this->l['Sirv account'],
                'order' => '30',
                'default'    => '',
                'label' => $this->l['Secret key'],
                'description'    => '',
                'type' => 'text',
                'scope' => 'module',
                'password'  =>true,
                'autocomplete'=> 'off'
            ),
            'network' => array(
                'hidden' => true,
                'id' => 'network',
                'group' => $this->l['Optimisation settings'],
                'order' => '20',
                'default' => 'CDN',
                'label' => $this->l['Network'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'CDN',
                    'DIRECT'
                ),
                'valuesLabels'  => array(
                    // $this->l(
                    //     'CDN - files served from Sirv\'s global content delivery network (recommended when live).'
                    // ),
                    // $this->l(
                    //     'DIRECT - files served from Germany (recommended when testing).'
                    // )
                    '',
                    ''
                ),
                'scope' => 'module'
            ),




            'image_scaling' => array(
                'id' => 'image_scaling',
                'group' => $this->l['Image optimisation'],
                'order' => '28',
                'default' => '1',
                'label' => $this->l['Image scaling'],
                'description' =>
                $this->l['Serve perfectly scaled images to suit each users device, as recommended by Google PageSpeed Insights.'],
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),
            'add_width_height' => array(
                'id' => 'add_width_height',
                'group' => $this->l['Image optimisation'],
                'order' => '28',
                'default' => '0',
                'label' => $this->l['Add width/height'],
                'description' =>$this->l['Add width/height attributes to img tags, if missing'].'.',
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),
            'images_cms' => array(
                'id' => 'images_cms',
                'group' => $this->l['Image optimisation'],
                'order' => '28',
                'default' => '1',
                'label' => $this->l['Serve CMS images'],
                'description' => '',
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),
            'profile' => array(
                'id' => 'profile',
                'group' => $this->l['Image optimisation'],
                'order' => '35',
                'default' => '',
                'label' => $this->l['Image profile'],
                'description' =>
                    sprintf(
                        $this->l['Profiles can add watermarks, text overlays and other %seffects%s.'].
                        ' '.
                        $this->l['Create or change profiles on your %sSirv profiles page%s.'],
                        '<a href="https://sirv.com/help/resources/dynamic-imaging/" target="_blank">',
                        '</a>',
                        '<a href="https://my.sirv.com/#/profiles/" target="_blank">',
                        '</a>'
                    )
                ,
                'allowEmpty' => true,
                'type' => 'array',
                'subType' => 'select',
                'values' => array(),
                'scope' => 'module'
            ),
            'quality' => array(
                'id' => 'quality',
                'group' => $this->l['Image optimisation'],
                'order' => '75',
                'default' => '',
                'label' => $this->l['Image quality'],
                'description' =>
                    defined('PS_ADMIN_DIR')?
                    $this->l['Your Sirv default quality is specified in your'].' '.
                    '<a target="_blank" href="https://my.sirv.com/#/profiles/?preview=%2FProfiles%2F'.$profileName.
                    '.profile">'.$profileName.' '.$this->l['profile'].'</a>.':
                    ''
                ,
                'allowEmpty' => true,
                'type' => 'array',
                'subType' => 'select',
                'values' => array(),
                'scope' => 'module'
            ),



            'images_lazyload' => array(
                'id' => 'images_lazyload',
                'group' => $this->l['Lazy loading'],
                'order' => '28',
                'default' => '1',
                'label' => $this->l['Lazy loading'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                  1,
                  2,
                  0
                ),
                'valuesLabels'  => array(
                    $this->l['Sirv JS lazy loading'],
                    $this->l['Native lazy loading'],
                    $this->l['Disabled'],
                ),
                'scope' => 'module'
            ),
            'lazy_placeholder' => array(
                'id' => 'lazy_placeholder',
                'group' => $this->l['Lazy loading'],
                'order' => '28',
                'default' => 'disabled',
                'label' => $this->l['Lazy placeholder'],
                'description' => $this->l['Display background while image loads.'],
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                  'image',
                  'blurred',
                  'background',
                  'disabled',
                ),
                'valuesLabels'  => array(
                    $this->l['ImageLazy'],
                    $this->l['Blurred image'],
                    $this->l['Grey background'],
                    $this->l['Disabled']
                ),
                'scope' => 'module'
            ),
            'excludefiles_from_lazy' => array(
                'id' => 'excludefiles_from_lazy',
                'group' => $this->l['Lazy loading'],
                'order' => '40',
                'default' => '',
                'label' => $this->l['Exclude images'],
                'description' => $this->l['You can enter full URLs and the domain will be stripped'].'<br/>'.
                $this->l['Use * to specify all images at a certain path'].'.<br/><br/>',
                'allowEmpty' => 1,
                'type' => 'textarea',
                'scope' => 'module'
            ),






            'serve_jscss' => array(
                'id' => 'serve_jscss',
                'group' => $this->l['JS & CSS optimisation'],
                'order' => '27',
                'default' => '0',
                'label' => $this->l['Serve JS, CSS and media files'],
                'description' =>
                $this->l['Speed-up your website by serving static files from Sirv\'s fast CDN (recommended).'],
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),
            'minify_jscss' => array(
                'id' => 'minify_jscss',
                'group' => $this->l['JS & CSS optimisation'],
                'order' => '28',
                'default' => '0',
                'label' => $this->l['Minify JS/CSS'],
                'description' => $this->l['Minify JS/CSS files to reduce size up to 80%.'].
                ' '.
                $this->l['Occasionally, JS files can break, so test your site after enabling.'],
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),
            'delete_old_js_css' => array(
                'id' => 'delete_old_js_css',
                'group' => $this->l['JS & CSS optimisation'],
                'order' => '28',
                'default' => '0',
                'label' => $this->l['Delere old files'],
                'description' => $this->l['You can delete old files from Sirv which have been uploaded by the module more than 90 days and have mot been accessed more than 60 days.'],
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),

            'folder' => array(
                'id' => 'folder',
                'group' => $this->l['Optimisation settings'],
                'order' => '30',
                'allowEmpty' => '',
                'default' => 'prestashop',
                'label' => $this->l['Folder name on Sirv'],
                'description' =>
                    $this->l['Images will be copied to this folder, autocreated in your Sirv account.'].
                    ' '.$this->l['To use the same Sirv account on multiple PrestaShop sites'].', '.
                    $this->l['use a different folder name for each site.'],
                'type' => 'text',
                'hidden' => true,
                'scope' => 'module'
            ),
            


            'use_smv' => array(
                'id' => 'use_smv',
                'group' => $this->l['Sirv Media Viewer'],
                'order' => '40',
                'default' => '0',
                'label' => $this->l['Enable Sirv Media Viewer'],
                'description' =>
                // $this->l(
                //     'Replace your product image gallery with Sirv Media Viewer on the product details page'
                // ),
                '',
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('Yes', 'No'),
                'valuesLabels'  => array($this->l['Yes'], $this->l['No']),
                'scope' => 'module'
            ),

            'view_file_folder' => array(
                'id' => 'view_file_folder',
                'group' => $this->l['Content from Sirv'],
                'order' => '100',
                'default' => 'products/{product-sku}',
                'label' => $this->l['Folder structure on Sirv'],
                'description' => $this->l['Possible variables'].
                ': {product-name}, {product-id}, {product-sku}',
                'type' => 'text',
                'scope' => 'module'
            ),
            'view_file_folder_combination' => array(
                'id' => 'view_file_folder_combination',
                'group' => $this->l['Content from Sirv'],
                'order' => '120',
                'default' => 'products/{product-sku}-{combination-sku}',
                'label' => $this->l['Folder structure for combinations'],
                'description' =>
                    $this->l['Possible variables'].': {product-id}, {product-sku}, {combination-sku}'
                ,
                'type' => 'text',
                'scope' => 'module'
            ),

            'view_file_cache_ttl' => array(
                'id' => 'view_file_cache_ttl',
                'group' => $this->l['Content from Sirv'],
                'order' => '130',
                'default' => '1440',
                'label' => $this->l['Cache TTL'],
                'description' => $this->l['Time after which the cache entry expires'],
                'values'  => array(
                    0,
                    1,
                    15,
                    60,
                    4*50,
                    24*60,
                    24*60*7,
                    1*30*24*60
                ),
                'valuesLabels'  => array(
                    $this->l['Disable cache (suitable for testing only)'],
                    '1 '.$this->l['minute'],
                    '15 '.$this->l['minutes'],
                    '1 '.$this->l['hour'],
                    '4 '.$this->l['hours'],
                    '1 '.$this->l['day'].' ('.$this->l['default setting, for typical use'].')',
                    '7 '.$this->l['days'],
                    '1 '.$this->l['month']
                ),
                'type' => 'array',
                'subType' => 'select',
                'scope' => 'module'
            ),


            'smv_combinations' => array(
                'id' => 'smv_combinations',
                'group' => $this->l['Order of content'],
                'order' => '60',
                'default' => 'ALL',
                'label' => $this->l['Content to show'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'ALL',
                    'SPECIFIC'
                ),
                'valuesLabels'  => array(
                    $this->l['Show all images'],
                    $this->l['Show images for current combination only'],
                ),
                'scope' => 'module'
            ),
            'add_view_files' => array(
                'id' => 'add_view_files',
                'group' => $this->l['Order of content'],
                'order' => '80',
                'default' => '1',
                'hidden' => true,
                'label' => $this->l['Extra content from Sirv'],
                'description' => $this->l['Add 360 spins, videos and images to your product gallery'],
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('1', '0'),
                'valuesLabels'  => array(
                    $this->l['Yes'],
                    $this->l['No'],
                ),
                'scope' => 'module'
            ),

            'add_view_files_method' => array(
                'id' => 'add_view_files_method',
                'group' => $this->l['Order of content'],
                'order' => '135',
                'default' => 'AFTER',
                'label' => $this->l['Order of content'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'BEFORE',
                    'AFTER',
                    'REPLACE',
                ),
                'valuesLabels'  => array(
                    $this->l['Sirv content first'],
                    $this->l['PrestaShop content first'],
                    $this->l['Only show Sirv content'],
                ),
                'scope' => 'module'
            ),

            'smv_spin_position' => array(
                'id' => 'smv_spin_position',
                'group' => $this->l['Order of content'],
                'order' => '145',
                'default' => '',
                'label' => $this->l['360 spin position'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'FIRST',
                    'SECOND',
                    'THIRD',
                    'LAST',
                    ''
                ),
                'valuesLabels'  => array(
                    $this->l['First'],
                    $this->l['Second'],
                    $this->l['Third'],
                    $this->l['Last'],
                    $this->l['Alphabetical'],                    
                ),
                'allowEmpty' => true,
                'scope' => 'module'
            ),
            'smv_video_position' => array(
                'id' => 'smv_video_position',
                'group' => $this->l['Order of content'],
                'order' => '155',
                'default' => '',
                'label' => $this->l['Video position'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'FIRST',
                    'SECOND',
                    'THIRD',
                    'LAST',
                    '',
                ),
                'valuesLabels'  => array(
                    $this->l['First'],
                    $this->l['Second'],
                    $this->l['Third'],
                    $this->l['Last'],
                    $this->l['Alphabetical'],                    
                ),
                'allowEmpty' => true,
                'scope' => 'module'
            ),

            'smv_pin_video' => array(
                'id' => 'smv_pin_video',
                'group' => $this->l['Order of content'],
                'order' => '255',
                'default' => 'NO',
                'label' => $this->l['Pin video(s)'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'LEFT',
                    'RIGHT',
                    'NO'
                ),
                'valuesLabels'  => array(
                    $this->l['Left'],
                    $this->l['Right'],
                    $this->l['No']
                ),
                'allowEmpty' => true,
                'scope' => 'module'
            ),

            'smv_pin_spin' => array(
                'id' => 'smv_pin_spin',
                'group' => $this->l['Order of content'],
                'order' => '260',
                'default' => 'NO',
                'label' => $this->l['Pin spin(s)'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'LEFT',
                    'RIGHT',
                    'NO'
                ),
                'valuesLabels'  => array(
                    $this->l['Left'],
                    $this->l['Right'],
                    $this->l['No']
                ),
                'allowEmpty' => true,
                'scope' => 'module'
            ),

            'smv_pin_by_mask' => array(
                'id' => 'smv_pin_by_mask',
                'group' => $this->l['Order of content'],
                'order' => '270',
                'default' => 'NO',
                'label' => $this->l['Pin images by file mask'],
                'description' => $this->l['Pin a file with a particular filename'],
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'LEFT',
                    'RIGHT',
                    'NO'
                ),
                'valuesLabels'  => array(
                    $this->l['Left'],
                    $this->l['Right'],
                    $this->l['No']
                ),
                'allowEmpty' => true,
                'scope' => 'module'
            ),

            'smv_pin_mask' => array(
                'id' => 'smv_pin_mask',
                'group' => $this->l['Order of content'],
                'order' => '275',
                'default' => '',
                'label' => $this->l['File mask'],
                'description' => 'Enter characters to match either part or all of the filename e.g. dimensions.jpg or dimensions',
                'type' => 'text',
                'allowEmpty' => true,
                'scope' => 'module'
            ),


            'smv_layout' => array(
                'id' => 'smv_layout',
                'group' => $this->l['Design and experience'],
                'order' => '10',
                'default' => 'GALLERY',
                'label' => $this->l['Layout'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'GALLERY',
                    'GRID_1_COLUMN',
                    'GRID_2_COLUMN',
                    'GRID_3_COLUMN',
                    'GRID_2_COLUMN_FI',
                    'GRID_3_COLUMN_FI',
                ),
                'valuesLabels'  => array(
                    $this->l['Gallery'],
                    $this->l['1 column'],
                    $this->l['2 columns'],
                    $this->l['3 columns'],
                    $this->l['1 then 2'],
                    $this->l['1 then 3'],
                ),
                'allowEmpty' => true,
                'scope' => 'module'
            ),

            'smv_spacing' => array(
                'id' => 'smv_spacing',
                'group' => $this->l['Design and experience'],
                'order' => '59',
                'default' => '20',
                'label' => $this->l['Spacing between items in the grid'],
                'description' => '',
                'type' => 'text',
                'scope' => 'module',
                'allowEmpty' => true
            ),


            'smv_ratio' => array(
                'id' => 'smv_ratio',
                'group' => $this->l['Design and experience'],
                'order' => '59',
                'default' => 'Auto',
                'label' => $this->l['Aspect ratio of item(s)'],
                'description' => '',

                'type' => 'array',
                'subType' => 'radio',
                'values'  => array(
                    'Auto',
                    '1/1',
                    '4/3',
                    '16/9',
                    '3/4',
                    '9/16',
                    'Custom',
                ),

                'scope' => 'module',
                'allowEmpty' => true
            ),

            'smv_ratio_custom' => array(
                'id' => 'smv_ratio_custom',
                'group' => $this->l['Design and experience'],
                'order' => '59',
                'default' => '1/1',
                'label' => $this->l['Custom Aspect ratio'],
                'description' => 'e.g. 1/1, 16/9, 0.75',
                'type' => 'text',
                'scope' => 'module',
                'allowEmpty' => true
            ),

            'smvoptions' => array(
                'id' => 'smvoptions',
                'group' => $this->l['Design and experience'],
                'order' => '140',
                'default' => '',
                'label' => $this->l['Script options'],
                'description' => '',
                'type' => 'textarea',
                'scope' => 'module'
            ),
            
            'smvcss' => array(
                'id' => 'smvcss',
                'group' => $this->l['Design and experience'],
                'order' => '150',
                'default' => '',
                'label' => $this->l['CSS styling'],
                'description' => '',
                'type' => 'textarea',
                'scope' => 'module'
            ),

            'image_zoom' => array(
                'id' => 'image_zoom',
                'group' => $this->l['Design and experience'],
                'order' => '160',
                'default' => '1',
                'label' => $this->l['Enable image zoom'],
                'description' => '',
                'type' => 'array',
                'subType' => 'checkbox',
                'values'  => array('1', '0'),
                'valuesLabels'  => array(
                    $this->l['Yes'],
                    $this->l['No'],
                ),
                'scope' => 'module'
            ),
            'smv_max_height' => array(
                'id' => 'smv_max_height',
                'group' => $this->l['Design and experience'],
                'order' => '180',
                'default' => '',
                'label' => $this->l['Max height'],
                'description' => $this->l['Maximum height of viewer, including thumbnails'],
                'type' => 'text',
                'scope' => 'module',
                'allowEmpty' => true
            ),


            'sirvjscomponents' => array(
                'id' => 'sirvjscomponents',
                'group' => $this->l['Optimisation settings'],
                'order' => '140',
                'default' => 'lazyimage,zoom,spin,hotspots,video,gallery,model',
                'values'  => array(
                    'lazyimage','zoom','spin','hotspots','video','gallery','model'
                ),
                'valuesLabels'  => array(
                    $this->l['Lazy & responsive images'],
                    $this->l['Image zoom'],
                    $this->l['360 spin'],
                    $this->l['Hotspots'],
                    $this->l['Video streaming'],
                    $this->l['Gallery (still images, YouTube & Vimeo videos)'],
                    'Model',
                ),
                'label' => $this->l['Sirv JS features'],
                'description' => '',
                'allowEmpty' => 1,
                'type' => 'array',
                'subType' => 'checkbox',
                'scope' => 'module'
            ),

            'includesirvjs' => array(
                'id' => 'includesirvjs',
                'group' => $this->l['Optimisation settings'],
                'order' => '140',
                'default' => 'detect',
                'values'  => array(
                    'detect','allpages','nopages'
                ),
                'valuesLabels'  => array(
                    $this->l['Detect - add script only to pages that require it'],
                    $this->l['All pages - always add script (select this if images are not loading)'],
                    $this->l['No pages - don\'t add script (may break responsive images)'],
                ),
                'label' => $this->l['Include Sirv JS'],
                'description' => '',
                'type' => 'array',
                'subType' => 'radio',
                'scope' => 'module'
            ),



            'httpusername' => array(
                'id' => 'httpusername',
                'group' => $this->l['Optimisation settings'],
                'order' => '30',
                'default' => '',
                'label' => $this->l['Username'],
                'description' => '',
                'type' => 'text',
                'allowEmpty' => true,
                'hidden' => false,
                'scope' => 'module'
            ),
            'httppassword' => array(
                'id' => 'httppassword',
                'group' => $this->l['Optimisation settings'],
                'order' => '30',
                'default' => '',
                'label' => $this->l['Password'],
                'description' => '',
                'type' => 'text',
                'allowEmpty' => true,
                'hidden' => false,
                'scope' => 'module'
            ),




            

            'delete_thumbnails' => array(
                'id' => 'delete_thumbnails',
                'group' => $this->l['Synchronization settings'],
                'order' => '40',
                'default' => 'Keep',
                'label' => $this->l['Delete images created by PrestaShop'],
                'description' =>

                    defined('PS_ADMIN_DIR')?

                    $this->l['Save disk space on your PrestaShop server by deleting resized images.'].'<br/>'.
                    sprintf(
                        $this->l['%sThumbnails can be regenerated%s if needed.'],
                        '<a href="'. $link->getAdminLink('AdminImages', true).'" target="_blank">',
                        '</a>'
                    )
                    :''
                ,
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array('Delete', 'Keep'),
                'valuesLabels'  => array(
                    $this->l['Delete'],
                    $this->l['Keep']
                ),
                'scope' => 'module'
            ),

            'delete_images' => array(
                'id' => 'delete_images',
                'group' => $this->l['Synchronization settings'],
                'order' => '40',
                'default' => 'Delete',
                'label' => $this->l['Delete images from Sirv server'],
                'description' =>
                    defined('PS_ADMIN_DIR')?
                    $this->l['Automatically delete image from Sirv server when it deletes in Prestashop.']
                    :''
                ,
                'type' => 'array',
                'subType' => 'radio',
                'values'  => array('Delete', 'Keep'),
                'valuesLabels'  => array(
                    $this->l['Delete'],
                    $this->l['Keep']
                ),
                'scope' => 'module'
            ),
            'excludefiles' => array(
                'id' => 'excludefiles',
                'group' => $this->l['Synchronization settings'],
                'order' => '150',
                'default' => '',
                'label' => $this->l['Exclude files/folders'],
                'description' => $this->l['You can enter full URLs and the domain will be stripped'].'.<br/>'.
                $this->l['Use * to specify all files at a certain path'].'.',
                'allowEmpty' => 1,
                'type' => 'textarea',
                'scope' => 'module'
            ),
            'excludepages' => array(
                'id' => 'excludepages',
                'group' => $this->l['Synchronization settings'],
                'order' => '150',
                'default' => '',
                'label' => $this->l['Exclude pages'],
                'description' => $this->l['You can enter full URLs and the domain will be stripped'].'.<br/>'.
                $this->l['Use * to specify all pages at a certain path'].'.',
                'allowEmpty' => 1,
                'type' => 'textarea',
                'scope' => 'module'
            ),

            'customcss' => array(
                'id' => 'customcss',
                'group' => $this->l['Miscellaneous'],
                'order' => '140',
                'default' => '',                
                'label' => $this->l['Custom CSS'],
                'description' => $this->l['Custom CSS description'],
                'allowEmpty' => 1,
                'type' => 'textarea',                
                'scope' => 'module'
            ),

        );

        $q = '80';

        if (defined('PS_ADMIN_DIR') && Configuration::get('SIRV_ACTIVE') && $this->getSirvURL(true) != '') {
            $profile = 'Default';
            if (Configuration::get('SIRV_PROFILE')!='') {
                $profile = Configuration::get('SIRV_PROFILE');
            }

            $profileContents = @Tools::file_get_contents($this->getSirvURL(true).'/Profiles/'.$profile.'.profile');

            if (!empty($profileContents)) {
                $profileJSON = @json_decode($profileContents);
                if (isset($profileJSON->image) &&
                    isset($profileJSON->image->quality) && (int)$profileJSON->image->quality) {
                    $q = (int)$profileJSON->image->quality;
                }
            }
        }
    
        $options['quality']['values'][] = 0;
        $options['quality']['valuesLabels'][] = $this->l['Your Sirv default'].' ('.$q.'%)';
        for ($i=100; $i >= 1; $i--) {
            $s = '';
            switch ($i) {
                case 100:
                    $s = ' - '.$this->l['Extreme quality (huge filesize)'];
                    break;
                case 90:
                    $s = ' - '.$this->l['Very high quality'];
                    break;
                case 80:
                    $s = ' - '.$this->l['High quality (Sirv recommended default)'];
                    break;
                case 65:
                    $s = ' - '.$this->l['Medium quality'];
                    break;
                case 40:
                    $s = ' - '.$this->l['Low quality'];
                    break;
                case 20:
                    $s = ' - '.$this->l['Very low quality'];
                    break;
            }
            $options['quality']['values'][] = $i;
            $options['quality']['valuesLabels'][] = $i.'%'.$s;
        }
    


        return $options;
    }
}
