<?php
/**
* NOTICE OF LICENSE
*
* This file is licenced under the Software License Agreement.
* With the purchase or the installation of the software in your application
* you accept the licence agreement.
*
* You must not modify, adapt or create derivative works of this source code
*
*  @author    Sirv Limited <support@sirv.com>
*  @copyright Copyright (c) 2017-2023 Sirv Limited. All rights reserved
*  @license   https://www.magictoolbox.com/license/
*/
if (!defined('_PS_VERSION_')) {
    exit;
}

class Sirv extends Module
{

    public $isPrestaShop15x = false;
    public $isPrestaShop155x = false;
    /* PrestaShop v1.6 or above */
    public $isPrestaShop16x = false;
    public $isPrestaShop17x = false;
    public $isSmarty3 = false;
    public $s3client = false;
    public $APIClient = false;
    public $tool = false;
    public $imageTypes = false;
    public $params;
    public $ImageTypes;
    public $defaultClient = array(
        'clientId'      =>  'HsZHPqXM2VSDGSkiVUVbLGAvxhC',
        'clientSecret'  =>  'YTzRmo70TIwwyBsyP/pBUZ6IbzokDrZwSFJ4JIGvM6V6CYeZXnbH8CgQFj72FyCR/VMImC8FicJTJZtjuHft3Q=='
    );
    private $translationString;

    public function __construct()
    {

        $this->name = 'sirv';
        $this->tab = 'Tools';
        $this->version = '5.2.0';
        $this->author = 'Sirv';

        $this->module_key = 'd19552b1bede04a66680267c9321149f';

        $this->ImageTypes = $this->getImageTypes();

        $this->bootstrap = true;

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_ );

        parent::__construct();

        $this->displayName = $this->l('Sirv - Image CDN, Optimization and Responsive Images');
        $this->description = $this->l('Speed up your website with Sirv CDN. Enjoy faster loading and better Google ranking. Outstanding image optimisation and resizing - every PrestaShop store performs better. Highly optimized images; WebP; JS & CSS minification; 360-degree spins; image zoom; higher conversions; better SEO. The best PrestaShop CDN.');

        $this->confirmUninstall =
        $this->l('All Sirv settings will be deleted. Are you certain you want to uninstall this module?');

        $this->translationString = array(
            'original' => array(
                'This month',
                'January',
                'February',
                'March',
                'April',
                'May',
                'June',
                'July',
                'August',
                'September',
                'October',
                'November',
                'December'
            ),
            'translated' => array(
                $this->l('This month'),
                $this->l('January'),
                $this->l('February'),
                $this->l('March'),
                $this->l('April'),
                $this->l('May'),
                $this->l('June'),
                $this->l('July'),
                $this->l('August'),
                $this->l('September'),
                $this->l('October'),
                $this->l('November'),
                $this->l('December')
            ),
        );



        if (!empty($_REQUEST['sirvfixdb'])) {
            $this->fixDB();
            die('DB has been updated');
        }

        $this->isPrestaShop15x = version_compare(_PS_VERSION_, '1.5', '>=');
        $this->isPrestaShop155x = version_compare(_PS_VERSION_, '1.5.5', '>=');
        $this->isPrestaShop16x = version_compare(_PS_VERSION_, '1.6', '>=');
        $this->isPrestaShop17x = version_compare(_PS_VERSION_, '1.7', '>=');

        $this->isSmarty3 = $this->isPrestaShop15x || Configuration::get('PS_FORCE_SMARTY_2') === '0';

        $this->params = $this->getParams();

        if ($this->isPrestaShop16x) {
            $this->tab = 'others';
        }
    }

    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        $headerHookID = Hook::getIdByName('displayHeader');

        $updateHookName = false;
        $isPrestaShop1451 = version_compare(_PS_VERSION_, '1.4.5.1', '>=');
        if ($isPrestaShop1451) {
            $updateHookName = 'afterSaveProduct';
            if ($this->isPrestaShop15x) {
                $updateHookName = 'actionProductSave';
                if ($this->isPrestaShop16x) {
                    $updateHookName = 'actionProductUpdate';
                    if ($this->isPrestaShop17x) {
                        $updateHookName = 'actionProductSave';
                    }
                }
            }
        }

        foreach ($this->params as $param) {
            if ('SIRV_SIRVJSCOMPONENTS' == 'SIRV_'.Tools::strtoupper($param['id'])) {
                continue;
            }
            if (Configuration::get('SIRV_'.Tools::strtoupper($param['id']))=='') {
                Configuration::updateValue(
                    'SIRV_'.Tools::strtoupper($param['id']),
                    $param['default']
                );
            }
        }


        $this->registerHook('hookbackOfficeFooter');

        if (!parent::install() ||
            !$this->registerHook($this->isPrestaShop15x ? 'displayHeader' : 'header') ||
            !$this->registerHook('displayAdminProductsMainStepLeftColumnMiddle') ||
            
            
            !$this->registerHook('actionFrontControllerSetMedia') ||
            !$this->registerHook('actionBeforeAjaxDie') ||

            !$this->registerHook('displayAdminProductsExtra') ||
            
            !($updateHookName && $this->registerHook($updateHookName)) ||

            !($this->updatePosition($headerHookID, 0, 1) || true) ||

            !$this->installDB() ||
            !$this->installTabs()
        ) {
            return false;
        }


        return true;
    }

    const ADMIN_CONTROLLERS = array(
        array(
            'name' => 'Sirv Optimisation',
            'visible' => true,
            'class_name' => 'AdminSirvPsConfig',
            'parent_class_name' => 'IMPROVE',
            'icon' => 'filter_drama',
        ),
    );

    public function installTabs()
    {
        foreach (static::ADMIN_CONTROLLERS as $adminTab) {
            if (false === $this->installTab($adminTab)) {
                return false;
            }
        }

        return true;
    }

    public function installTab(array $tabData)
    {
        $tab = new Tab();
        $tab->id_parent = Tab::getIdFromClassName($tabData['parent_class_name']);
        $tab->name = array_fill_keys(array_values(Language::getIDs(false)), $tabData['name']);
        $tab->class_name = $tabData['class_name'];
        // Makes the module accessible in the Admin Controller, see ModuleAdminControllerCore constructor.
        $tab->module = $this->name;
        $tab->active = (bool) $tabData['visible'];
        $tab->icon = $tabData['icon'];

        return $tab->save();
    }

    public function uninstallTabs()
    {
        foreach (static::ADMIN_CONTROLLERS as $adminTab) {
            if (false === $this->uninstallTab($adminTab)) {
                $this->_errors[] = 'Failed to uninstall all tabs.';
                return false;
            }
        }

        return true;
    }

    public function uninstallTab(array $tabData)
    {
        $tabId = Tab::getIdFromClassName($tabData['class_name']);
        $tab = new Tab($tabId);

        if (false === (bool) $tab->delete()) {
            return false;
        }

        return true;
    }

    private function installDB()
    {
        if (!Db::getInstance()->Execute('CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'sirv_images` (
                                  `id` int(11) NOT NULL AUTO_INCREMENT,
                                  `path` varchar(255) DEFAULT NULL,
                                  `size` int(10) DEFAULT NULL,
                                  `sirvpath` varchar(255) DEFAULT NULL,
                                  `timestamp` datetime DEFAULT NULL,
                                  `timestamp_synced` datetime DEFAULT NULL,
                                  `item_id` varchar(45) DEFAULT NULL,
                                  `status` enum(\'NEW\',\'PROCESSING\',\'SYNCED\',\'FAILED\') DEFAULT NULL,
                                  `type` enum(\'PRODUCT\',\'MAGIC360\',\'MAGICSLIDESHOW\',\'MAGICSCROLL\',\'CUSTOM\',
                                  \'CATEGORY\') DEFAULT NULL,
                                  `checks` TINYINT UNSIGNED NULL DEFAULT 0,
                                  `msg` varchar(200) DEFAULT NULL,
                                PRIMARY KEY (`id`),
                                UNIQUE KEY `index_unique` (`item_id`,`type`),
                                KEY `index_item_id` (`item_id`),
                                KEY `index_status` (`status`)
                                ) ENGINE=InnoDB DEFAULT CHARSET=utf8;') ||
            !Db::getInstance()->query('CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'sirv_views_cache` (
                    `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
                    `path` varchar(255) DEFAULT NULL,
                    `res` text,
                    `status` enum(\'FAILED\',\'EMPTY\',\'SUCCESS\') DEFAULT NULL,
                    `time` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
                    PRIMARY KEY (`id`)
                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8;') ||
            !Db::getInstance()->query('CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'sirv_video` (
                    `id_product` int unsigned NOT NULL,
                    `data` mediumtext,
                    PRIMARY KEY (`id_product`)
                    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;') ||
            !Db::getInstance()->query('CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'sirv_assets` (
                    `id_product` int unsigned NOT NULL,
                    `data` mediumtext,
                    PRIMARY KEY (`id_product`)
                    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;') ||
            !$this->fixDB() || !$this->fillDB()
        ) {
            return false;
        }

        return true;
    }

    private function fixDB()
    {
        $res = true;

        $columns = array(
            'status' => array(
                'ADD COLUMN `status` enum(\'NEW\',\'PROCESSING\',\'SYNCED\') DEFAULT NULL',
                'UPDATE `'._DB_PREFIX_.'sirv_images` SET status = \'SYNCED\''
            ),
            'type' => array(
                'ADD COLUMN `type` enum(\'PRODUCT\',\'MAGIC360\',\'MAGICSLIDESHOW\',\'MAGICSCROLL\',\'CUSTOM\',
                \'CATEGORY\') DEFAULT NULL',
                'UPDATE `'._DB_PREFIX_.'sirv_images` SET type = \'PRODUCT\''
            ),
            'checks' => array(
                'ADD COLUMN `checks` TINYINT UNSIGNED NULL DEFAULT 0'
            ),
        );
        foreach ($columns as $column => $sqls) {
            if (!$this->ifColumnExists($column, 'sirv_images')) {
                $res = $res && Db::getInstance()->query('ALTER TABLE `'._DB_PREFIX_.'sirv_images` '.$sqls[0]);
                if (!empty($sqls[1])) {
                    $res = $res && Db::getInstance()->query($sqls[1]);
                }
            }
        }

        Db::getInstance()->query(
            'DELETE t1 FROM '._DB_PREFIX_.'sirv_images t1
            INNER JOIN '._DB_PREFIX_.'sirv_images t2
            WHERE t1.id < t2.id AND t1.item_id= t2.item_id AND t1.type= t2.type'
        );

        $indexes = array();
        $results = Db::getInstance()->ExecuteS('SHOW INDEX FROM '._DB_PREFIX_.'sirv_images');
        if ($results) {
            foreach ($results as $row) {
                $indexes[$row['Key_name']] = $row['Key_name'];
            }
        }

        if (empty($indexes['index_unique'])) {
            Db::getInstance()->query(
                'ALTER TABLE '._DB_PREFIX_.'sirv_images 
                ADD UNIQUE INDEX `index_unique` (`item_id` ASC, `type` ASC)'
            );
        }
        if (empty($indexes['index_item_id'])) {
            Db::getInstance()->query(
                'ALTER TABLE '._DB_PREFIX_.'sirv_images 
                ADD INDEX `index_item_id` (`item_id`)'
            );
        }
        Db::getInstance()->query(
            'ALTER TABLE '._DB_PREFIX_.'sirv_images 
            CHANGE COLUMN `status` 
            `status` ENUM(\'NEW\', \'PROCESSING\', \'SYNCED\', \'FAILED\') NULL DEFAULT NULL'
        );
        
        Db::getInstance()->query(
            'ALTER TABLE '._DB_PREFIX_.'sirv_images 
            CHANGE COLUMN `type` `type` 
            ENUM(\'PRODUCT\', \'MAGIC360\', \'MAGICSCROLL\', \'MAGICSLIDESHOW\', \'CUSTOM\',
            \'CATEGORY\') NULL DEFAULT NULL'
        );
        
        return $res;
    }

    private function ifColumnExists($column, $table)
    {
        $sql = 'SHOW COLUMNS FROM `'._DB_PREFIX_.$table.'` LIKE \''.$column.'\'';
        return Db::getInstance()->ExecuteS($sql);
    }

    public function uninstall()
    {
        if (!parent::uninstall() ||
            !$this->uninstallTabs()
            /* || !$this->uninstallDB()*/) {
            Configuration::updateValue('SIRV_ACTIVE', '0');
            return false;
        }
        return true;
    }

    public function disable($force_all = false)
    {
        if (!parent::disable($force_all)) {
            return false;
        }
        return true;
    }

    private function uninstallDB()
    {
        $result = true;

        foreach ($this->getParams() as $param) {
            $result = $result && Configuration::deleteByName('SIRV_'.Tools::strtoupper($param['id']));
        }

        return $result;
    }

    public function hookActionBeforeAjaxDie($params)
    {
        $this->processFetchQueue();
    }

    public function hookbackOfficeFooter($params)
    {

        if (defined('PS_ADMIN_DIR') &&
            Configuration::get('SIRV_ACTIVE') && Module::isEnabled($this->name)
        ) {
            if (method_exists($this->context->controller, 'addJquery')) {
                $this->context->controller->addJquery();
            }
            if (empty(Tools::getValue('id_product'))) {
                return '';
            }

            $product = new Product((int)Tools::getValue('id_product'), true);

            $productImages = $product->getImages($params['cookie']->id_lang);

            foreach ($productImages as $i => $image) {
                if (is_array($product->link_rewrite)) {
                    $product->link_rewrite = reset($product->link_rewrite);
                }
                $imgPath = _PS_PROD_IMG_DIR_.Image::getImgFolderStatic($image['id_image']).$image['id_image'].'.jpg';
                $product_link_rewrite = $product->link_rewrite;

                $productImages[$i]['url'] = $this->syncImage(
                    $product_link_rewrite,
                    $image['id_image'],
                    '',
                    '',
                    $imgPath,
                    'PRODUCT',
                    true
                );

                if (empty($productImages[$i]['url'])) {
                    unset( $productImages[$i] );
                }

            }

            $GLOBALS['productImages'] = $productImages;

            ob_start();
            require('templates/admin-images-fix.tpl.php');
            $html = ob_get_clean();
            
            unset($GLOBALS['productImages']);
            
            return $html.$this->hookBackOfficeFooter_video();
        }
        return $this->hookBackOfficeFooter_video();
    }

    public function hookDisplayAdminProductsMainStepLeftColumnMiddle($params)
    {

        if (defined('PS_ADMIN_DIR') &&
            version_compare(_PS_VERSION_, '1.7.0', '>=') &&
            Configuration::get('SIRV_ACTIVE') && Module::isEnabled($this->name)
        ) {
            if (method_exists($this->context->controller, 'addJquery')) {
                $this->context->controller->addJquery();
            }

            if (empty($params['id_product'])) {
                return '';
            }

            $product = new Product((int)$params['id_product'], true);

            $productImages = $product->getImages($params['cookie']->id_lang);

            $images = array();

            foreach ($productImages as $i => $image) {
                if (is_array($product->link_rewrite)) {
                    $product->link_rewrite = reset($product->link_rewrite);
                }
                $imgPath = _PS_PROD_IMG_DIR_.Image::getImgFolderStatic($image['id_image']).$image['id_image'].'.jpg';
                $product_link_rewrite = $product->link_rewrite;

                $productImages[$i]['url'] = $this->syncImage(
                    $product_link_rewrite,
                    $image['id_image'],
                    '',
                    '',
                    $imgPath,
                    'PRODUCT',
                    true
                );

                $i_url = Image::getImgFolderStatic($image['id_image']).$image['id_image'].'-'.
                ImageType::getFormattedName('small');

                $images[$i_url] = $productImages[$i]['url'].'?w=130&h=130&canvas.width=130&canvas.height=130';

                if (empty($productImages[$i]['url'])) {
                    unset( $productImages[$i] );
                    unset( $images[$i_url] );
                }

            }

            $GLOBALS['productImages'] = $productImages;
            $GLOBALS['images'] = $images;

            ob_start();
            require('templates/admin-images-fix-style.tpl.php');
            unset($GLOBALS['productImages']);
            unset($GLOBALS['images']);
            return ob_get_clean();
        }
        return '';
    }

    public function hookActionFrontControllerSetMedia($params)
    {
        if ('product' === $this->context->controller->php_self
            && !$this->isPageExcluded()
            && Configuration::get('SIRV_USE_SMV') == 1
            && Configuration::get('SIRV_ACTIVE') == 1) {
            if ($this->isPrestaShop17x) {
                $this->context->controller->registerStylesheet(
                    'sirv-product-css',
                    'modules/sirv/views/css/product.css',
                    [
                      'media' => 'all',
                      'priority' => 200,
                    ]
                );
                $this->context->controller->registerJavascript(
                    'sirv-product-js',
                    'modules/sirv/views/js/product.js',
                    [
                      'priority' => 200,
                    ]
                );
            } elseif ($this->isPrestaShop16x) {
                $this->context->controller->addJS(($this->_path).'views/js/product.js');
                $this->context->controller->addCSS(($this->_path).'views/css/product.css', 'all');
            }
        }
    }


    public function hookHeader($params)
    {
        if (Configuration::get('SIRV_ACTIVE') == 0 || $this->isPageExcluded()) {
            return;
        }

        $smarty = &$GLOBALS['smarty'];

        if ($this->isSmarty3) {
            $smarty->registerFilter('output', array(Module::getInstanceByName('sirv'), 'parseTemplateStandard'));
        } else {
            $smarty->register_outputfilter(array(Module::getInstanceByName('sirv'), 'parseTemplateStandard'));
        }

        $GLOBALS['url'] = $this->getSirvURL(false);
        $GLOBALS['sirvjs'] = (Configuration::get('SIRV_IMAGES_LAZYLOAD') || Configuration::get('SIRV_IMAGE_SCALING')) ||
        'product' === $this->context->controller->php_self && Configuration::get('SIRV_USE_SMV') == 1;

        ob_start();
        require('templates/header.tpl.php');

        unset($GLOBALS['url']);
        unset($GLOBALS['sirvjs']);

        return ob_get_clean();
    }

    private function sortImagesByCover($arr)
    {
        usort($arr, function ($a, $b) {
            if ($a['cover'] == $b['cover']) {
                return 0;
            }
            return ($a['cover'] > $b['cover']) ? -1 : 1;
        });
        return $arr;
    }

    private function addURL2Assets($assets, $url)
    {
        foreach ($assets as $i => $a) {
            $assets[$i]->url = $url.'/'.$a->name;
        }
        return $assets;
    }

    protected function sortableString($str)
    {
        $str = preg_replace('/\\.[^.\\s]+$/', '', $str);
        $str = strtolower($str);

        return preg_replace_callback(
            '/(\d+)/is',
            function($d) {
                return str_pad($d[0], 6, '0', STR_PAD_LEFT);
            },
            $str
        );
    }

    protected function compareAssets($a, $b)
    {
        if ($a->type != 'image' && $b->type == 'image') {
            return  1;
        } else if ($a->type == 'image' && $b->type != 'image') {
            return -1;
        } else if ($a->type != 'image' && $b->type != 'image') {
            return 0;
        }

        $file_name_a = self::sortableString($a->name);
        $file_name_b = self::sortableString($b->name);

        return strcmp($file_name_a, $file_name_b);
    }    

    public function getViewFileContents($folderName, $var, $replacments, $force = false)
    {

        $path = str_replace($var, $replacments, $folderName);

        $url = $this->getSirvURL().preg_replace(
            '/^\/(.*)\/$/ims',
            '$1',
            str_replace('{folder-name}', Configuration::get('SIRV_FOLDER'), $path)
        );

        $sql = "SELECT *, MINUTE(TIMEDIFF(NOW(), `time`)) as ttl ".
        "FROM "._DB_PREFIX_."sirv_views_cache WHERE path = '".pSQL($path).
        "' OR path = '".pSQL(str_replace('{folder-name}', Configuration::get('SIRV_FOLDER'), $path))."'";

        $view = Db::getInstance()->getRow($sql);

        if (!empty($view) && (int)$view['ttl'] > (int)Configuration::get('SIRV_VIEW_FILE_CACHE_TTL')
            || (int)Configuration::get('SIRV_VIEW_FILE_CACHE_TTL') == 0) {
            $force = true;
        }


        if (!empty($view) && !empty($view['id']) && empty($force)) {
            $res = array();
            switch ($view['status']) {
                case 'FAILED':
                case 'EMPTY':
                    break;
                case 'SUCCESS':
                    $view = json_decode($view['res']);

                    usort($view->assets, array($this,'compareAssets') );

                    $res = $this->addURL2Assets($view->assets, $url);
                    break;
            }
            return $res;
        } else {
            //$res = @Tools::file_get_contents($url.'.view?info');
            $res = $this->getViewInfo($url.'.view?info');
            $view = @json_decode($res);

            if (json_last_error() != JSON_ERROR_NONE) {
                $view = json_decode('{"assets":{}}');
                $view->assets = [];
            }

            usort($view->assets, array($this,'compareAssets') );

            if ($force) {
                $sql = "DELETE FROM "._DB_PREFIX_."sirv_views_cache WHERE path = '".pSQL($path).
                "' OR path = '".pSQL(str_replace('{folder-name}', Configuration::get('SIRV_FOLDER'), $path))."'";
                Db::getInstance()->Execute($sql);
            }

            if (is_object($view) && !empty($view->assets) && count($view->assets)) {
                Db::getInstance()->insert('sirv_views_cache', array(
                    'path'      =>  pSQL($path),
                    'res'       =>  pSQL(json_encode($view)),
                    'status'    =>  pSQL('SUCCESS')
                ), true, false, Db::REPLACE);
                return $this->addURL2Assets($view->assets, $url);
            } else {
                Db::getInstance()->insert('sirv_views_cache', array(
                    'path'      =>  pSQL($path),
                    'res'       =>  pSQL(''),
                    'status'    =>  pSQL(is_object($view)?'EMPTY':'FAILED')
                ), true, false, Db::REPLACE);
            }
        }
        return [];
    }


    public function getExtraAssets($id_product, $force = true)
    {

        $assets = [];

        $res = [];

        $urls = $this->loadProductAssetsData($id_product);

        foreach($urls as $url => $url_data) {

            $sql = "SELECT *, MINUTE(TIMEDIFF(NOW(), `time`)) as ttl ".
            "FROM "._DB_PREFIX_."sirv_views_cache WHERE path = '".pSQL($url)."'";

            $asset = Db::getInstance()->getRow($sql);

            if (!empty($asset) && (int)$asset['ttl'] > (int)Configuration::get('SIRV_VIEW_FILE_CACHE_TTL')
                || (int)Configuration::get('SIRV_VIEW_FILE_CACHE_TTL') == 0) {
                $force = true;
            }

            if (!empty($asset) && !empty($asset['id']) && empty($force)) {
                switch ($asset['status']) {
                    case 'FAILED':
                    case 'EMPTY':
                        break;
                    case 'SUCCESS':
                        $asset = json_decode($asset['res']);
                        $res[] = (array)$asset;
                        break;
                }                
            } else {
                $urlExists = $this->checkIfURLExists($url);
                $info = [];
                if ($urlExists) {
                    if (empty($this->APIClient)) {
                        $this->APIClient = $this->getAPIClient();
                    }
                    $asset = $this->APIClient->getFileStat(preg_replace('/^https?:\/\/.*?\/(.*)$/ims','$1', $this->decodeURLParts($url)));
                    if (!is_object($asset)) {
                        continue;
                    } else {
                        $asset = (array) $asset;
                    }
                } else {
                    $asset = [];
                }

                $asset['url'] = $url;

                if ($force) {
                    $sql = "DELETE FROM "._DB_PREFIX_."sirv_views_cache WHERE path = '".pSQL($url)."'";
                    Db::getInstance()->Execute($sql);
                }

                if ($urlExists || $urlExists && is_object($asset)) {
                    Db::getInstance()->insert('sirv_views_cache', array(
                        'path'      =>  pSQL($url),
                        'res'       =>  pSQL(json_encode($asset)),
                        'status'    =>  pSQL('SUCCESS')
                    ), true, false, Db::REPLACE);
                    $res[] = (array) $asset;
                } else {
                    Db::getInstance()->insert('sirv_views_cache', array(
                        'path'      =>  pSQL($url),
                        'res'       =>  pSQL(''),
                        'status'    =>  pSQL('FAILED')
                    ), true, false, Db::REPLACE);
                }
            }

            if (!empty($res) && is_array($res) && count($res)) {                    
                
                $a = $res[count($res)-1];


                if (empty($a['type'])) {
                    if (!empty($a['contentType']) && preg_match('/image/ims', $a['contentType'])) {
                        $a['type'] = 'image';
                    }
                    if (!empty($a['contentType']) && preg_match('/video/ims', $a['contentType'])) {
                        $a['type'] = 'video';
                    }
                    if (empty($a['type']) && preg_match('/\.(mp4|mov)/ims', $url)) {
                        $a['type'] = 'video';
                    }
                    if (empty($a['type']) && preg_match('/\.spin/ims', $url)) {
                        $a['type'] = 'spin';
                    }
                    if (empty($a['type']) && preg_match('/\.(jpg|jpeg|png|webp|avif|gif|svg)/ims', $url)) {
                        $a['type'] = 'image';
                    }
                }
                

                $res[count($res)-1] = (object) $a;

            }
        }

        return $res;
    }

    private function getViewInfo($url) {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_REFERER, _PS_BASE_URL_SSL_);
        $result = curl_exec($curl);
        curl_close($curl);
        return $result;
    }

    private function searchInAssetsArray($elm, $arr)
    {
        foreach ($arr as $key => $v) {
            if ($v['url'] == $elm['url']) {
                return $key;
            }
        }
        return false;
    }

    public function parseTemplateStandard($output, $smarty, $fromOverride = false)
    {

        if (!$fromOverride) {
            return $output;
        }


        if (empty($smarty)) {
            $smarty = $this->context->smarty;
        }

        if ($this->isSmarty3) {
            $template_resource = @$smarty->template_resource;
            if (empty($template_resource)) {
                $template_resource = '';
            }
            $currentTemplate = Tools::substr(basename($template_resource), 0, -4);
            if ($currentTemplate == 'breadcrumb') {
                $currentTemplate = 'product';
            } elseif ($currentTemplate == 'pagination') {
                $currentTemplate = 'category';
            }
        } else {
            $currentTemplate = $smarty->currentTemplate;
        }

        if ('product' === $this->context->controller->php_self &&
            Configuration::get('SIRV_ACTIVE') &&
            !$this->isPageExcluded() &&
            Configuration::get('SIRV_USE_SMV') == 1 /*&& $currentTemplate == 'product'*/) {
            $product_id = (int)Tools::getValue('id_product');
            $product = new Product((int)$product_id, true, $this->context->language->id);
            $images_data = $product->getImages($this->context->language->id);

            $images = array();
            $coverNum = 0;
            $needToResync = false;

            if (count($images_data)) {
                foreach ($images_data as $i => $image) {
                    if ($image['cover']) {
                        $coverNum = $i;
                    }
                    $image_url = $this->context->link->getImageLink($product->link_rewrite, $image['id_image'], '');
                    $needToResync = $needToResync || (strpos($image_url, $this->getSirvURL()) === false);
                    $images[$image['id_image']] = array(
                        'id' => $image['id_image'],
                        'type' => 'image',
                        'source' => 'prestashop',
                        'cover' => (bool)$image['cover'],
                        'alt' => (!empty($image['legend']))?$image['legend']:$product->name,
                        'url'  => $image_url
                    );
                }
            }

            if ($needToResync) {
                $this->processFetchQueue();
                // $images = array();
                // $coverNum = 0;

                // if (count($images_data)) {
                //     foreach ($images_data as $i => $image) {

                //         $image_db = Db::getInstance()->getRow('SELECT * FROM '._DB_PREFIX_.'image WHERE id_image = '.$image['id_image']);
                //         $image['cover'] = $image_db['front_cover'];

                //         if ($image['cover']) {
                //             $coverNum = $i;
                //         }
                //         $images[$image['id_image']] = array(
                //             'id' => $image['id_image'],
                //             'type' => 'image',
                //             'source' => 'prestashop',
                //             'cover' => (bool)$image['cover'],
                //             'alt' => (!empty($image['legend']))?$image['legend']:$product->name,
                //             'url'  => $this->context->link->getImageLink($product->link_rewrite, $image['id_image'], '')
                //         );
                //     }
                // }
            }



            

            $sirvAssetID = 0;

            if (true || Configuration::get('SIRV_ADD_VIEW_FILES')) {
                $assets = $this->getViewFileContents(
                    Configuration::get('SIRV_VIEW_FILE_FOLDER'),
                    array(
                        '{product-sku}',
                        '{product-name}',
                        '{product-id}',
                    ),
                    array(
                        $product->reference,
                        $product->name,
                        $product_id,
                    )
                );

                if (!empty($extraAssets) && $extraAssets) {
                    $extraAssets = [];
                }
                if ($assets) {
                    $assets = [];
                }

                $extraAssets = $this->getExtraAssets($product_id);

                if (is_array($extraAssets)) {
                    $assets = array_merge($assets, $extraAssets);
                }

                $newAssets = array();
                if (!empty($assets)) {
                    $profile = Configuration::get('SIRV_PROFILE');


                    foreach ($assets as $i => $a) {

                        if ($a->type != 'image' && $a->type != 'spin' && $a->type != 'video') {
                            continue;
                        }
                        $newAssets['sirv-'.$sirvAssetID] = array(
                            'id' => 'sirv-'.$sirvAssetID,
                            'source' => 'sirv',
                            'type' => $a->type,
                            'cover' => false,
                            'alt' => $product->name,
                            'url'  => $a->url.(!empty($profile)?'?profile='.urldecode($profile):''),
                        );
                        $sirvAssetID++;
                    }

                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'REPLACE') {
                        $images = $newAssets;
                    }
                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'BEFORE') {
                        $images = array_merge($newAssets, $images);
                    }
                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'AFTER') {
                        $images = array_merge($images, $newAssets);
                    }
                }
            }

            $combinations = $product->getAttributeCombinations($this->context->language->id);

            $combinations_ = array();
            foreach ($combinations as $c) {
                $combinations_[$c['id_product_attribute']] = $c;
            }
            $combinations = $combinations_;

            $combination_urls = array();

            foreach ($combinations as $c) {
                $url = $this->context->link->getProductLink(
                    $c['id_product'],
                    null,
                    null,
                    null,
                    null,
                    null,
                    $c['id_product_attribute'],
                    Configuration::get('PS_REWRITING_SETTINGS'),
                    false,
                    true
                );
                
                $hash = preg_replace('/.*\#(.*?)/ims', '$1', $url);
                $hash_arr = explode('/', $hash);
                sort($hash_arr);
                $combination_urls[ join('/', $hash_arr) ] = array(
                    'id' => $c['id_product_attribute']
                );
            }

            $combination_images = $product->getCombinationImages($this->context->language->id);

            $c_images = array();

            $needToResync = false;

            if (!empty($combination_images) && count($combination_images)) {
                foreach ($combination_images as $combination_id => $c) {
                    $c_images[$combination_id] = array();
                    foreach ($c as $image) {
                        $c_images[$combination_id][$image['id_image']] =
                        $this->context->link->getImageLink($product->link_rewrite, $image['id_image'], '');
                        $needToResync = $needToResync || (strpos($c_images[$combination_id][$image['id_image']], $this->getSirvURL()) === false);

                    }
                }
                if ($needToResync) {
                    $this->processFetchQueue();
                    // foreach ($combination_images as $combination_id => $c) {
                    //     $c_images[$combination_id] = array();
                    //     foreach ($c as $image) {
                    //         $c_images[$combination_id][$image['id_image']] =
                    //         $this->context->link->getImageLink($product->link_rewrite, $image['id_image'], '');
    
                    //     }
                    // }
                }
            }

            foreach ($combinations as $comb) {

                $combination_id = $comb['id_product_attribute'];

                $assets = $this->getViewFileContents(
                    Configuration::get('SIRV_VIEW_FILE_FOLDER_COMBINATION'),
                    array(
                        '{product-sku}',
                        '{product-id}',
                        '{combination-sku}',
                    ),
                    array(
                        $product->reference,
                        $product_id,
                        $comb['reference'],
                    )
                );

                if (!empty($assets)) {
                    $newAssets = array();

                    $profile = Configuration::get('SIRV_PROFILE');

                    foreach ($assets as $i => $a) {
                        if ($a->type != 'image' && $a->type != 'spin' && $a->type != 'video') {
                            continue;
                        }

                        $id = $sirvAssetID;

                        $asset = array(
                            'id' => 'sirv-'.$sirvAssetID,
                            'source' => 'sirv',
                            'type' => $a->type,
                            'cover' => false,
                            'alt' => $product->name,
                            'url'  => $a->url.(!empty($profile)?'?profile='.urldecode($profile):''),
                            'disabled' =>  Configuration::get('SIRV_SMV_COMBINATIONS')=='SPECIFIC',
                            'combination_id' => $combination_id
                        );

                        $newAssets['sirv-'.$sirvAssetID] = $asset;

                        $sirvAssetID++;
                    }

                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'REPLACE') {
                        $images = $newAssets;
                    }
                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'BEFORE') {
                        $images = array_merge($newAssets, $images);
                    }
                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'AFTER') {
                        $images = array_merge($images, $newAssets);
                    }


                    if (empty($c_images[$combination_id])) {
                        $c_images[$combination_id] = array();
                    }
                    
                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'REPLACE') {
                        $c_images[$combination_id] = $newAssets;
                    }
                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'BEFORE') {
                        $c_images[$combination_id] = $newAssets + $c_images[$combination_id];
                    }
                    if (count($newAssets) && Configuration::get('SIRV_ADD_VIEW_FILES_METHOD') == 'AFTER') {
                        $c_images[$combination_id] = $c_images[$combination_id] + $newAssets;
                    }
                }
            }

            $productVideos = $this->loadProductVideoData($product_id);

            if (count($productVideos)) {
                foreach (array_keys($productVideos) as $i => $v) {
                    $images[] = array(
                        'id' => 'ext-video-'.$i,
                        'type' => 'video',
                        'source' => 'external',
                        'cover' => false,
                        'url'  => $v
                    );
                }
            }

            if (count($images)) {
                $pos = 1;

                $posExtraShifts = array(
                    'FIRST' => 0,
                    'SECOND' => 0,
                    'THIRD' => 0,
                    'LAST' => 0,
                );

                foreach ($images as $i => $image) {
                    if ($image['source'] == 'sirv' && ($image['type'] == 'spin' || $image['type'] == 'video')) {
                        $position = Configuration::get('SIRV_SMV_'.Tools::strtoupper($image['type']).'_POSITION');
                        if ($position == 'FIRST') {
                            $new_pos = 0 + $posExtraShifts['FIRST'];
                            $posExtraShifts['FIRST'] += '0.0001';
                        } elseif ($position == 'SECOND') {
                            $new_pos = 1.5 + $posExtraShifts['SECOND'];
                            $posExtraShifts['SECOND'] += '0.0001';
                        } elseif ($position == 'THIRD') {
                            $new_pos = 2.5 + $posExtraShifts['THIRD'];
                            $posExtraShifts['THIRD'] += '0.0001';
                        } elseif ($position == 'LAST') {
                            $new_pos = 1000 + $posExtraShifts['LAST'];
                            $posExtraShifts['LAST'] += '0.0001';
                        } else {
                            $new_pos = $pos++;
                        }
                        $images[$i]['pos'] = $new_pos++;
                    } else {
                        $images[$i]['pos'] = $pos++;
                    }
                }

                uasort($images, function ($a, $b) {
                    if ($a['pos'] == $b['pos']) {
                        return 0;
                    }
                    return ($a['pos'] < $b['pos']) ? -1 : 1;
                });

                $c_images_first = $c_images;
                foreach ($c_images_first as $id => $c) {
                    $c_images_first[$id] = array_keys($c)[0];
                }

                $ajaxURL = $this->context->link->getModuleLink('sirv', 'ajax', array());

                $GLOBALS['ps17'] = $this->isPrestaShop17x;
                $GLOBALS['ps16'] = $this->isPrestaShop16x;
                
                $coverNum = 0;
                foreach ($images as $i) {
                    if ($i['cover'] == 1) {
                        break;
                    }
                    $coverNum++;
                }

                $layout_options = '';

                if (preg_match('/GRID_/ims', Configuration::get('SIRV_SMV_LAYOUT'))) {
                    $layout_options_array = ['layout.type:grid'];                        

                    if (Configuration::get('SIRV_SMV_LAYOUT') == 'GRID_1_COLUMN') {
                        $layout_options_array[] = 'layout.grid.columns:1';
                    }
                    if (Configuration::get('SIRV_SMV_LAYOUT') == 'GRID_2_COLUMN' || Configuration::get('SIRV_SMV_LAYOUT') == 'GRID_2_COLUMN_FI' ) {
                        $layout_options_array[] = 'layout.grid.columns:2';
                    }
                    if (Configuration::get('SIRV_SMV_LAYOUT') == 'GRID_3_COLUMN' || Configuration::get('SIRV_SMV_LAYOUT') == 'GRID_3_COLUMN_FI' ) {
                        $layout_options_array[] = 'layout.grid.columns:2';
                    }

                    if ( Configuration::get('SIRV_SMV_RATIO') == 'Custom' && preg_match('/^([\.0-9]{1,}| *[0-9]{1,} *\/ * [0-9]{1,} *)$/ims', Configuration::get('SIRV_SMV_RATIO_CUSTOM'))) {
                        $layout_options_array[] = 'layout.aspectratio:'.Configuration::get('SIRV_SMV_RATIO_CUSTOM');
                    } elseif (Configuration::get('SIRV_SMV_RATIO') != 'Custom' && Configuration::get('SIRV_SMV_RATIO')!='' ) {
                        $layout_options_array[] = 'layout.aspectRatio:'.strtolower(Configuration::get('SIRV_SMV_RATIO'));                        
                    }
                    
                    
                    if (preg_match('/^[0-9]{1,}$/ims', Configuration::get('SIRV_SMV_SPACING'))) {
                        $layout_options_array[] = 'layout.grid.gap:'.Configuration::get('SIRV_SMV_SPACING');
                    }
                    $layout_options = join(';', $layout_options_array);

                    if (Configuration::get('SIRV_SMV_LAYOUT') == 'GRID_2_COLUMN_FI' || Configuration::get('SIRV_SMV_LAYOUT') == 'GRID_3_COLUMN_FI') {
                        $GLOBALS['SIRV_SMV_FULLWIDTH_FIMAGE'] = Configuration::get('SIRV_SMV_COLUMNS');
                    }
                }


                $GLOBALS['ajaxURL'] = $ajaxURL;
                $GLOBALS['layout_options'] = $layout_options;
                $GLOBALS['product_id'] = $product_id;
                $GLOBALS['combination_urls'] = $combination_urls;
                $GLOBALS['c_images'] = $c_images;
                $GLOBALS['c_images_first'] = $c_images_first;
                $GLOBALS['coverNum'] = $coverNum;
                $GLOBALS['images'] = $images;
                $GLOBALS['getSirvURL'] = $this->getSirvURL();
                
                $GLOBALS['pin_video'] = Configuration::get('SIRV_SMV_PIN_VIDEO');
                $GLOBALS['pin_spin'] = Configuration::get('SIRV_SMV_PIN_SPIN');
                $GLOBALS['pin_image'] = Configuration::get('SIRV_SMV_PIN_BY_MASK');
                $GLOBALS['pin_image_mask'] = Configuration::get('SIRV_SMV_PIN_MASK');



                ob_start();
                require('templates/sirv-media-viewer.tpl.php');
                $html = ob_get_clean();

                unset($GLOBALS['ajaxURL']);
                unset($GLOBALS['layout_options']);
                unset($GLOBALS['product_id']);
                unset($GLOBALS['combination_urls']);
                unset($GLOBALS['c_images']);
                unset($GLOBALS['c_images_first']);
                unset($GLOBALS['coverNum']);
                unset($GLOBALS['images']);
                unset($GLOBALS['getSirvURL']);
                unset($GLOBALS['pin_video']);
                unset($GLOBALS['pin_spin']);
                unset($GLOBALS['pin_image']);
                unset($GLOBALS['pin_image_mask']);
                

                if ($this->isPrestaShop17x) {
                    $mainImagePattern = '<div\b[^>]*?\bclass\s*+=\s*+"[^"]*?\bimages-container\b[^"]*+"[^>]*+>'.
                                        '('.
                                        '(?:'.
                                            '[^<]++'.
                                            '|'.
                                            '<(?!/?div\b|!--)'.
                                            '|'.
                                            '<!--.*?-->'.
                                            '|'.
                                            '<div\b[^>]*+>'.
                                                '(?1)'.
                                            '</div\s*+>'.
                                        ')*+'.
                                        ')'.
                                        '</div\s*+>';
                } else {
                    $thumbsPattern =    '(<div\b[^>]*?(?:\bid\s*+=\s*+"(?:views_block|thumbs_list)"|\b'.
                                        'class\s*+=\s*+"[^"]*?\bimage-additional\b[^"]*+")[^>]*+>)'.
                                        '('.
                                        '(?:'.
                                            '[^<]++'.
                                            '|'.
                                            '<(?!/?div\b|!--)'.
                                            '|'.
                                            '<!--.*?-->'.
                                            '|'.
                                            '<div\b[^>]*+>'.
                                                '(?2)'.
                                            '</div\s*+>'.
                                        ')*+'.
                                        ')'.
                                        '</div\s*+>';
                    $output = preg_replace(
                        '#'.$thumbsPattern.'#ims',
                        '<div class="hidden-important">$0</div>',
                        $output
                    );
                    $output = preg_replace(
                        '#<p class="resetimg.*?</p>#ims',
                        '<div class="hidden-important">$0</div>',
                        $output
                    );

                    $mainImagePattern = '(<div\b[^>]*?(?:\bid\s*+=\s*+"image-block"|\b'.
                                        'class\s*+=\s*+"[^"]*?\bimage\b[^"]*+")[^>]*+>)'.
                                        '('.
                                        '(?:'.
                                            '[^<]++'.
                                            '|'.
                                            '<(?!/?div\b|!--)'.
                                            '|'.
                                            '<!--.*?-->'.
                                            '|'.
                                            '<div\b[^>]*+>'.
                                                '(?2)'.
                                            '</div\s*+>'.
                                        ')*+'.
                                        ')'.
                                        '</div\s*+>';
                }

                $output = preg_replace(
                    '#'.$mainImagePattern.'#ims',
                    '<div class="hidden-important">$0</div>'.$html,
                    $output,
                    1
                );

                $arrowsPattern = '<div\b[^>]*?\bclass\s*+=\s*+"[^"]*?\bscroll-box-arrows\b[^"]*+"[^>]*+>'.
                                    '('.
                                    '(?:'.
                                        '[^<]++'.
                                        '|'.
                                        '<(?!/?div\b|!--)'.
                                        '|'.
                                        '<!--.*?-->'.
                                        '|'.
                                        '<div\b[^>]*+>'.
                                            '(?1)'.
                                        '</div\s*+>'.
                                    ')*+'.
                                    ')'.
                                    '</div\s*+>';
                $output = preg_replace('#'.$arrowsPattern.'#', '', $output);
            }
        }
        return $output;
    }

    public function getBaseUrl()
    {
        if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') {
            $protocol = ($_SERVER['HTTPS'] && $_SERVER['HTTPS'] != "off") ? "https" : "http";
        } else {
            $protocol = 'http';
        }
        return $protocol . "://" . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
    }

    private function clearCacheSirv()
    {
        Tools::clearSmartyCache();
        Tools::clearXMLCache();
        Media::clearCache();
        Tools::generateIndex();
        return true;
    }

    public function getContent()
    {
        $error = array();
        $output = '';

        $link = new Link();

        if (Tools::getValue('sirv_disconnect_user', '') == '1') {
            $url = ( !$this->isPrestaShop17x?_PS_BASE_URL_.__PS_BASE_URI__.basename(_PS_ADMIN_DIR_).'/':'' ).
            $link->getAdminLink('AdminModules', true).'&configure=sirv';
            Configuration::updateValue('SIRV_ACTIVE', '0');
            Configuration::updateValue('SIRV_CLIENT_ID', '');
            Configuration::updateValue('SIRV_CLIENT_SECRET', '');
            Configuration::updateValue('SIRV_TOKEN', '');
            Configuration::updateValue('SIRV_TOKEN_EXPIRE_TIME', '');
            Configuration::updateValue('SIRV_MUTE', '');
            Configuration::updateValue('SIRV_S3_BUCKET', '');
            Configuration::updateValue('SIRV_S3_ACCESS_KEY', '');
            Configuration::updateValue('SIRV_S3_SECRET_KEY', '');
            Configuration::updateValue('SIRV_ACCOUNT_EMAIL', '');
            Configuration::updateValue('SIRV_STAT', '');
            Configuration::updateValue('SIRV_CDN_URL', '');
            Tools::redirect($url);
            die();
        }

        $savedSettings = false;

        $this->s3client = $this->getS3Client();
        $this->APIClient = $this->getAPIClient();

        $SirvStatus = $this->APIClient->preOperationCheck();

        $this->params = $this->getParams();

        if ($SirvStatus) {
            $accountInfo = $this->APIClient->getAccountInfo();
            



            if (!empty($accountInfo)) {
                Configuration::updateValue('SIRV_S3_BUCKET', $accountInfo->alias);
                Configuration::updateValue(
                    'SIRV_NETWORK',
                    (!empty($accountInfo->cdnURL) ||
                    !empty($accountInfo->aliases->{$accountInfo->alias}->customDomain) ||
                    !empty($accountInfo->aliases->{$accountInfo->alias}->cdn))?'CDN':'DIRECT'
                );
                Configuration::updateValue('SIRV_CDN_URL', (!empty($accountInfo->cdnURL))?$accountInfo->cdnURL:'');
            }

            $storageInfo = $this->getStorageInfo();
            
            $storageInfo['traffic']['allowance_text'] =
            str_replace('&#8734', '∞', $storageInfo['traffic']['allowance_text']);
            $storageInfo['plan']['dataTransferLimit_text'] = str_replace(
                '&#8734',
                '∞',
                $storageInfo['plan']['dataTransferLimit_text']
            );

            $domains = array();

            if (!empty($accountInfo->cdnTempURL)) {
                $domains[$accountInfo->cdnTempURL] = $accountInfo->cdnTempURL;
            }
            if (!empty($accountInfo->alias)) {
                $domains[$accountInfo->alias.'.sirv.com'] = $accountInfo->alias.'.sirv.com';
            }
            if (!empty($accountInfo->aliases)) {
                foreach ($accountInfo->aliases as $a => $alias) {
                    $domain = !empty($alias->customDomain)?$alias->customDomain:$a.'.sirv.com';
                    $domains[$domain] = $domain;
                }
            }

            if (empty(Configuration::get('SIRV_DOMAIN')) && count($domains)) {
                Configuration::updateValue('SIRV_DOMAIN', $domains[array_keys($domains)[0]]);
            }

            $this->params['domain']['values'] = $domains;

            $isFreePlan = preg_match('/free/ims', $storageInfo['plan']['name']);

            if ($isFreePlan) {
                if (Tools::getValue('SIRV_SERVE_JSCSS') == 1 || Tools::getValue('SIRV_MINIFY_JSCSS') == 1) {
                    $error[] = $this->displayWarning(
                        $this->display(__FILE__, 'views/templates/admin/admin-warning-free.tpl')
                    );
                }
                $_POST['SIRV_SERVE_JSCSS'] = 0;
                $_POST['SIRV_MINIFY_JSCSS'] = 0;
            }
        }

        if (Tools::isSubmit('submit'.$this->name)) {
            $savedSettings = true;
            $emptyValues = array();

            $newFolder = trim(Tools::getValue('SIRV_FOLDER', ''));

            if ($newFolder != Configuration::get('SIRV_FOLDER')) {
                $this->APIClient->rename(
                    '/'.Configuration::get('SIRV_FOLDER'),
                    '/'.$newFolder
                );

                if (!$this->checkIfImageExists($newFolder.'/1x1.png')) {
                    $this->APIClient->uploadFile(
                        dirname(__FILE__).'/views/img/1x1.png',
                        '/'.$newFolder.'/1x1.png'
                    );
                }
            }

            $oldCDNStatus = Configuration::get('SIRV_NETWORK');
            if (Tools::getValue('http_authentication')) {
                $defaultTab = 'fieldset_image_synchronisation';
            }
            if (Tools::getValue('SIRV_DELETE_THUMBNAILS')!='') {
                $defaultTab = 'fieldset_image_synchronisation';
            }

            foreach ($this->params as $param_id => $param) {
                if (Tools::getValue('http_authentication') &&
                    !in_array($param_id, array('httppassword', 'httpusername'))) {
                    continue;
                }

                if (!empty($param['hidden'])) {
                    continue;
                }

                $val = (string)Tools::getValue('SIRV_'.Tools::strtoupper($param_id));

                if ($param_id == 'sirvjscomponents') {
                    $values = [];
                    foreach ($param['values'] as $v) {
                        if (!empty($_REQUEST['SIRV_SIRVJSCOMPONENTS_'.$v])) {
                            $values[] = $v;
                        }
                    }
                    $val = implode(',', $values);                    
                    Configuration::updateValue('SIRV_'.Tools::strtoupper($param_id), $val);
                    continue;
                }

                if (Tools::strtoupper($param_id) == 'SMVOPTIONS') {
                    $val = preg_replace('/<script[^>]*>(.*)<\/script>/ims', '$1', $val);
                }
                if (Tools::strtoupper($param_id) == 'SMVCSS') {
                    $val = preg_replace('/<style[^>]*>(.*)<\/style>/ims', '$1', $val);
                }
                if (Tools::strtoupper($param_id) == 'CUSTOMCSS') {
                    $val = preg_replace('/<style[^>]*>(.*)<\/style>/ims', '$1', $val);
                }

                if ((!$val || empty($val)) && (empty($param['allowEmpty'])) && !($val=='0')
                    && $param_id != 'smvoptions'
                    && $param_id != 'customcss'
                    && Tools::getIsset('SIRV_'.Tools::strtoupper($param_id))
                    && $param_id != 'smvcss') {
                    $emptyValues[] = $param['label'];
                } elseif (Tools::getIsset('SIRV_'.Tools::strtoupper($param_id))) {
                    Configuration::updateValue('SIRV_'.Tools::strtoupper($param_id), $val);
                }
            }

            if (!empty($emptyValues) && count($emptyValues)) {
                $error[] = $this->displayError(
                    $this->l('Enter your Sirv settings').' ('.join($emptyValues, ', ').')'
                );
            }

            $this->APIClient->setupS3Credentials(Configuration::get('SIRV_ACCOUNT_EMAIL'));

            // TODO: update only once
            $this->APIClient->setFolderOptions(Configuration::get('SIRV_FOLDER'), array('scanSpins' => false));

            if ($oldCDNStatus != Configuration::get('SIRV_NETWORK') && !empty($accountInfo)) {
                $this->APIClient->configCDN(Configuration::get('SIRV_NETWORK') == 'CDN', $accountInfo->alias);
            }

            if (Configuration::get('SIRV_SERVE_JSCSS')) {
                $this->APIClient->configFetching(
                    _PS_BASE_URL_.__PS_BASE_URI__,
                    true,
                    1 == Configuration::get('SIRV_MINIFY_JSCSS')
                );
                $this->APIClient->allowDangerousContentTypes();
            } else {
                $this->APIClient->configFetching('', false, false);
            }
        }

        if ($SirvStatus) {
            $accountInfo = $this->APIClient->getAccountInfo();
            if (!empty($accountInfo)) {
                Configuration::updateValue('SIRV_S3_BUCKET', $accountInfo->alias);
                Configuration::updateValue(
                    'SIRV_NETWORK',
                    (!empty($accountInfo->aliases->{$accountInfo->alias}->cdn))?'CDN':'DIRECT'
                );
                Configuration::updateValue('SIRV_CDN_URL', (!empty($accountInfo->cdnURL))?$accountInfo->cdnURL:'');
            }
        }

        if ($this->isMuted()) {
            $from_time = time();
            $minutes = round(abs(Configuration::get('SIRV_MUTE') - $from_time) / 60);
            $minutes .= " minute".( ($minutes>1)?'s':'' );

            $this->smarty->assign('muteTime', gmdate("H:i a (e)", Configuration::get('SIRV_MUTE')));
            $this->smarty->assign('muteMinutes', $minutes);

            $error[] = $this->displayError(
                $this->display(__FILE__, 'views/templates/admin/admin-warning-mute.tpl')
            );
            $SirvStatus = false;
        }

        if ($SirvStatus && Configuration::get('PS_DISABLE_OVERRIDES')) {
            $url = ( !$this->isPrestaShop17x?_PS_BASE_URL_.__PS_BASE_URI__.basename(_PS_ADMIN_DIR_).'/':'' ).
            $link->getAdminLink('AdminPerformance', true);

            $this->smarty->assign('overridesURL', $url);

            $error[] = $this->displayError(
                $this->display(__FILE__, 'views/templates/admin/admin-warning-overrides.tpl')
            );
        }

        if (preg_match(
            '/sirv/ims',
            Configuration::get('PS_MEDIA_SERVER_1').
            Configuration::get('PS_MEDIA_SERVER_2').
            Configuration::get('PS_MEDIA_SERVER_3')
        )) {
            $url = ( !$this->isPrestaShop17x?_PS_BASE_URL_.__PS_BASE_URI__.basename(_PS_ADMIN_DIR_).'/':'' ).
            $link->getAdminLink('AdminPerformance', true);
            $this->smarty->assign('mediaURL', $url);

            $error[] = $this->displayError(
                $this->display(__FILE__, 'views/templates/admin/admin-warning-media.tpl')
            );
        }

        if (!empty($error)) {
            $output .= join($error);
        } elseif ($savedSettings) {
            $url = ( !$this->isPrestaShop17x?_PS_BASE_URL_.__PS_BASE_URI__.basename(_PS_ADMIN_DIR_).'/':'' ).
            $link->getAdminLink('AdminPerformance', true);
            $this->smarty->assign('savedURL', $url);

            $output .= $this->displayConfirmation(
                $this->display(__FILE__, 'views/templates/admin/admin-warning-saved.tpl')
            );
        }

        if (Tools::isSubmit('cleanUpCache')) {
            $method = Tools::getValue('sirv_cleanupcache', 'quick');

            $defaultTab = 'fieldset_image_synchronisation';

            if ($method=='quick') {
                Db::getInstance()->delete(
                    'sirv_images'
                );
                $output .= $this->displayConfirmation("Sirv cache has been cleared.");
            } elseif ($method == 'failed') {
                Db::getInstance()->delete(
                    'sirv_images',
                    'status = "FAILED"'
                );
                $output .= $this->displayConfirmation("Failed images have been deleted from your Sirv cache").'.';
            } elseif ($method == 'full') {
                if (!$this->s3client->testConnection()) {
                    $output .= $this->displayError(
                        $this->l('Sirv S3 credentials declined. Please re-enter S3 credentials.')
                    );
                } else {
                    $sql = 'SELECT * FROM `'._DB_PREFIX_.'sirv_images`';
                    $result = Db::getInstance()->ExecuteS($sql);
                    foreach ($result as $row) {
                        if ($row['sirvpath'] != '') {
                            $this->deleteFile(Configuration::get('SIRV_FOLDER').'/'.$row['sirvpath']);
                        }
                        Db::getInstance()->delete(
                            'sirv_images',
                            'id = '.(int)$row['id']
                        );
                    }
                    $output .= $this->displayConfirmation($this->l('Your image cache has been cleared.'));
                }
            }
        }

        $this->context->controller->addCSS($this->_path .'views/css/admin.css', 'all');

        $this->context->controller->addJS('https://cdnjs.cloudflare.com/ajax/libs/riot/3.4.3/riot+compiler.min.js');
        $this->context->controller->addJS($this->_path .'views/js/admin.js', 'all');

        $link = new Link();

        $this->smarty->assign(
            'JS_sirv_module_dir',
            $this->context->link->getAdminLink('AdminModules').'&configure='.$this->name
        );
        $this->smarty->assign('JS_sirv_base_url', _PS_BASE_URL_.__PS_BASE_URI__);

        $this->smarty->assign('JS_sirv_domain', $this->getSirvURL(true));
        $this->smarty->assign('JS_sirv_module_url', $link->getAdminLink('AdminModules', true).'&configure=sirv');

        $this->smarty->assign(
            'JS_data_estimated',
            Configuration::get('SIRV_DATA_ESTIMATED') == 1 ? 1 : 0
        );

        $this->smarty->assign('sirv_folder', Configuration::get('SIRV_FOLDER'));

        $this->ajaxProcessSirvFillTable();

        

        if ($SirvStatus) {
            $output .= $this->display(__FILE__, 'views/templates/admin/prestui/ps-tags.tpl');

            $storageInfo['user'] = Configuration::get('SIRV_ACCOUNT_EMAIL');
            $storageInfo['domain'] = $this->getSirvURL(false);

            $this->smarty->assign('sirv_max_files_to_fetch', min( $storageInfo['limits_by_name']['fetch:file'], $storageInfo['limits_by_name']['rest:global'] ));

            $this->smarty->assign('storageInfo', $storageInfo);

            $this->smarty->assign('storageInfoShowAPI', false);

            $this->smarty->assign('limitsNames', array(
                'rest:global' => 'Total requests',
                // 'rest:post:files:fetch' => 'Fetch remote file(s) from URL',
                'rest:post:files:upload' => 'Upload file',
                'rest:post:files:delete' => 'Delete file',
                // 'rest:post:files:search' => 'Search files ',
                // 'rest:post:files:search:scroll' => 'Scroll through file search results',
                // 'rest:post:files:video2spin' => 'Convert video to spin',
                // 'rest:post:files:spin2video' => 'Convert spin to video',
                // 'rest:get:stats:http' => 'Bandwidth stats',
                // 'rest:get:stats:storage' => 'Storage stats',
                // // S3
                // 's3:global' => 'Total requests',
                // 's3:PUT' => 'Uploads',
                // 's3:GET' => 'Downloads',
                // 's3:DELETE' => 'Deletes',
                // // FTP
                // 'ftp:global' => 'Total requests',
                // 'ftp:STOR' => 'Uploads',
                // 'ftp:RETR' => 'Downloads',
                // 'ftp:DELE' => 'Deletes',
                // // Auto-Fetch
                'fetch:file' => 'Fetching files',
                // 'rest:post:account' => 'Update account',
                // 'rest:post:account:fetching' => 'Configure auto fetching',
                // 'rest:post:account:new' => 'Create new account',
                // 'rest:post:user:accounts' => 'List accounts',
                // 'rest:get:rest:credentials' => 'Get credentials',
                // 'rest:post:video:toSpin' => 'To spin conversation',
                // 'rest:post:upload:toSirv' => 'Upload to Sirv',
                // 'rest:post:upload:toSirv' => 'Upload to Sirv',
            ));

            $output .= $this->display(__FILE__, 'views/templates/admin/js.tpl');

            $cacheInfo = $this->getCacheInfo();

            $total_by_type_text = array();

            if ($cacheInfo['total_by_type']['PRODUCT']) {
                $total_by_type_text[$this->l('Product')] = $cacheInfo['total_by_type']['PRODUCT'];
            }
            if ($cacheInfo['total_by_type']['CATEGORY']) {
                $total_by_type_text[$this->l('Category')] = $cacheInfo['total_by_type']['CATEGORY'];
            }
            if ($cacheInfo['total_by_type']['MAGIC']) {
                $total_by_type_text[$this->l('Magic Toolbox')] = $cacheInfo['total_by_type']['MAGIC'];
            }
            if ($cacheInfo['total_by_type']['CUSTOM']) {
                $total_by_type_text[$this->l('CMS/Custom')] = $cacheInfo['total_by_type']['CUSTOM'];
            }

            $cacheInfo['total_by_type_text'] = $total_by_type_text;

            $this->smarty->assign('cacheInfo', $cacheInfo);

            $this->smarty->assign('sirv_form_action', $_SERVER['REQUEST_URI']);

            $this->smarty->assign(
                'totalPercentProgress',
                (int)(($cacheInfo['total_count']!=0)?
                (int)($cacheInfo['SYNCED']['count']+$cacheInfo['FAILED']['count'])/$cacheInfo['total_count']*100:
                0)
            );

            $this->smarty->assign(
                'totalSyncedAndFailed',
                (int)($cacheInfo['SYNCED']['count']+$cacheInfo['FAILED']['count'])
            );

            $data = Configuration::get('SIRV_DATA_ESTIMATED_JSON');

            if (!empty($data)) {
                $this->smarty->assign(
                    'sirv_estimate_data',
                    (array)json_decode($data)
                );
            }


            $this->smarty->assign(
                'progress',
                (($cacheInfo['total_count']!=0)?(int)$cacheInfo['SYNCED']['count']/$cacheInfo['total_count']*100:0)
            );

            $this->smarty->assign(
                'progress_failed',
                (($cacheInfo['total_count']!=0)?(int)$cacheInfo['FAILED']['count']/$cacheInfo['total_count']*100:0)
            );


            $this->smarty->assign(
                'progress_processing',
                (($cacheInfo['total_count']!=0)?(int)$cacheInfo['PROCESSING']['count']/$cacheInfo['total_count']*100:0)
            );

            $sirv_stat_last_update =  json_decode(Configuration::get('SIRV_STAT'), true);

            $this->smarty->assign(
                'sirv_stat_last_update',
                date("H:i:s", $sirv_stat_last_update['time'])
            );

            $cookie = new Cookie('ps');
            $iso_code = Language::getIsoById((int)$cookie->id_lang);

            $this->smarty->assign('admin_lang', $iso_code=='en'?'en':'fr');

            if (empty($defaultTab)) {
                $defaultTab = Tools::getValue('default_tab');
            }
            if (empty($defaultTab)) {
                $defaultTab = 'fieldset_0';
            }
            $this->smarty->assign('defaultTab', $defaultTab);

            
            
            $this->smarty->assign('moduleStatus', Configuration::get('SIRV_ACTIVE'));

            $sql = "SELECT COUNT(*) FROM "._DB_PREFIX_."sirv_views_cache";
            $viewsCacheAll = Db::getInstance()->getValue($sql);
            $this->smarty->assign('viewsCacheAll', $viewsCacheAll);

            $sql = "SELECT COUNT(*) FROM "._DB_PREFIX_."sirv_views_cache
            WHERE status = 'FAILED'";
            $viewsCacheMissed = Db::getInstance()->getValue($sql);
            $this->smarty->assign('viewsCacheMissed', $viewsCacheMissed);

            $this->smarty->assign(
                'content',
                $this->display(__FILE__, 'views/templates/admin/about.tpl').
                $this->displayForm().
                $this->display(__FILE__, 'views/templates/admin/cache-block.tpl').
                $this->display(__FILE__, 'views/templates/admin/your-account.tpl').
                $this->display(__FILE__, 'views/templates/admin/stats-block.tpl').
                $this->display(__FILE__, 'views/templates/admin/feedback-block.tpl')
            );

            $versionString = $this->getVersionString();

            $this->smarty->assign('versionString', $versionString['versionString']);

            return $output.
                $versionString['updateString'].
                (
                    (time()-Configuration::get('SIRV_INSTALLED_DATE')>60*60*24*30)?
                    $this->display(__FILE__, 'views/templates/admin/review-block.tpl'):
                    ''
                ).
                $this->display(__FILE__, 'views/templates/admin/admin.tpl');
        } else {
            $output .= $this->display(__FILE__, 'views/templates/admin/prestui/ps-tags.tpl');

            $this->smarty->assign(
                'default_account_name',
                preg_replace('/(?:https?\:\/\/)?(.*?)($|\.|\/).*'.'/ims', '$1', _PS_BASE_URL_)
            );
            return $output.
                (
                    (!$this->isMuted())?$this->display(__FILE__, 'views/templates/admin/init-block.tpl'):''
                );
        }
    }

    private function getProfilesList()
    {
        $profiles = $this->APIClient->getProfiles();
        if ($profiles && !empty($profiles->contents) && is_array($profiles->contents)) {
            $profilesList = array();
            foreach ($profiles->contents as $profile) {
                if (preg_match('/\.profile$/ims', $profile->filename) && $profile->filename != 'Default.profile') {
                    $profilesList[] = preg_replace('/(.*?)\.profile$/ims', '$1', $profile->filename);
                }
            }
            sort($profilesList);
            return $profilesList;
        }
        return array();
    }

    private function getChangeLog()
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://www.magictoolbox.com/changelog/sirvprestashop/');
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $response = curl_exec($ch);
        curl_close($ch);

        $responseObj = json_decode($response);
        if (is_object($responseObj)) {
            $GLOBALS['version'] = $this->version;
            $GLOBALS['responseObj'] = $responseObj;
            $GLOBALS['s1'] = $this->l('Your current version');

            ob_start();
            require('templates/changelog.tpl.php');
            unset($GLOBALS['version']);
            unset($GLOBALS['responseObj']);
            unset($GLOBALS['s1']);
            return ob_get_clean();
        }

        // $hostname = 'www.magictoolbox.com';
        // $errno = 0;
        // $errstr = '';
        // $path = 'changelog/sirvprestashop/';
        // $handle = @fsockopen('ssl://' . $hostname, 443, $errno, $errstr, 30);
        // if ($handle) {
        //     $response = '';
        //     $headers  = "GET /{$path} HTTP/1.1\r\n";
        //     $headers .= "Host: {$hostname}\r\n";
        //     $headers .= "Connection: Close\r\n\r\n";
        //     fwrite($handle, $headers);
        //     while (!feof($handle)) {
        //         $response .= fgets($handle);
        //     }
        //     fclose($handle);
        //     $response = Tools::substr($response, strpos($response, "\r\n\r\n") + 4);
        //     $responseObj = json_decode($response);            
        //     if (is_object($responseObj)) {
        //         $GLOBALS['version'] = $this->version;
        //         $GLOBALS['responseObj'] = $responseObj;
        //         $GLOBALS['s1'] = $this->l('Your current version');

        //         ob_start();
        //         require('templates/changelog.tpl.php');
        //         unset($GLOBALS['version']);
        //         unset($GLOBALS['responseObj']);
        //         unset($GLOBALS['s1']);
        //         return ob_get_clean();
        //     }
        // }
        return '';
    }

    private function getVersionString()
    {
        $versionString = $this->l('Module version').': ' . $this->version;

        $updateString = '';

        $hostname = 'www.magictoolbox.com';
        $errno = 0;
        $errstr = '';
        $path = 'api/platform/sirvprestashop/version/?t=' . time();
        $handle = @fsockopen('ssl://' . $hostname, 443, $errno, $errstr, 30);
        if ($handle) {
            $response = '';
            $headers  = "GET /{$path} HTTP/1.1\r\n";
            $headers .= "Host: {$hostname}\r\n";
            $headers .= "Connection: Close\r\n\r\n";
            fwrite($handle, $headers);
            while (!feof($handle)) {
                $response .= fgets($handle);
            }
            fclose($handle);
            $response = Tools::substr($response, strpos($response, "\r\n\r\n") + 4);
            $responseObj = json_decode($response);
            if (is_object($responseObj) && isset($responseObj->version)) {
                $match = array();
                if (preg_match('#v([0-9]++(?:\.[0-9]++)*+)#is', $responseObj->version, $match)) {
                    $GLOBALS['changelog'] = $this->getChangeLog();
                    $GLOBALS['showToolbar'] = (version_compare($this->version, $match[1], '<'));
                    $GLOBALS['latestVersion'] = $match[1];
                    $GLOBALS['s1'] = $this->l('Latest version');
                    $GLOBALS['s2'] = $this->l('What\'s new');
                    $GLOBALS['s3'] = $this->l('New version available');
                    $GLOBALS['s4'] = $this->l('Changelog');

                    ob_start();
                    require('templates/update-string.tpl.php');
                    $updateString = ob_get_clean();

                    unset($GLOBALS['changelog']);
                    unset($GLOBALS['latestVersion']);
                    unset($GLOBALS['s1']);
                    unset($GLOBALS['s2']);
                    unset($GLOBALS['s3']);
                    unset($GLOBALS['s4']);
                    unset($GLOBALS['showToolbar']);
                }
            }
        }
        return array(
            'versionString' => $versionString,
            'updateString' => $updateString,
        );
    }

    public function getStorageInfo($force = false)
    {

        $cached_stat = Configuration::get('SIRV_STAT');

        if (!empty($cached_stat)) {
            $storageInfo = @json_decode($cached_stat, true);

            if (json_last_error() == JSON_ERROR_NONE
                && is_array($storageInfo)
                && time() - $storageInfo['time'] < 60*60
                && !$force) {
                return $storageInfo['data'];
            }
        }

        $storageInfo = $this->APIClient->getStorageInfo();
        if (!empty($storageInfo['traffic']['traffic'])) {
            $traffic = array();
            foreach ($storageInfo['traffic']['traffic'] as $key => $t) {
                $traffic[
                    str_replace($this->translationString['original'], $this->translationString['translated'], $key)
                ] = $t;
            }
            $storageInfo['traffic']['traffic'] = $traffic;
        }


        Configuration::updateValue('SIRV_STAT', json_encode(array(
            'time'  => time(),
            'data'  => $storageInfo
        )));

        return $storageInfo;
    }

    public function ajaxProcessSirvRefreshImages()
    {
        Db::getInstance()->update('sirv_images', array(
            'status'            => pSQL('NEW'),
        ));
        return true;
    }

    public function ajaxProcessSirvSaveFolderName()
    {
        Configuration::updateValue('SIRV_FOLDER', Tools::getValue('folder'));
        die( );
    }


    public function syncImage($name, $ids, $type, $imgLink, $imgPath, $itemType = 'PRODUCT', $wait = false)
    {

        $cacheID = implode('', func_get_args());

        $imgLink = preg_replace('/(.*\.(png|jpg|gif))\?[a-z0-9]{1,}=.*$/ims', '$1', $imgLink);
        $imgPath = preg_replace('/(.*\.(png|jpg|gif))\?[a-z0-9]{1,}=.*$/ims', '$1', $imgPath);

        unset($type);

        if (!empty($GLOBALS['syncedImagesCache'][$cacheID])) {
            return $GLOBALS['syncedImagesCache'][$cacheID];
        }

        $imgPathShort = str_replace(_PS_ROOT_DIR_.'/', '', $imgPath);
        if ($itemType == 'PRODUCT') {
            $ids = (int)$ids;
            $product_id = Db::getInstance()->getValue(
                'SELECT id_product FROM '._DB_PREFIX_.'image WHERE id_image = '.(int)$ids
            );
            if ($product_id) {
                $product = new Product($product_id, false);
                $name = (is_array($product->link_rewrite))?$this->getFNE($product->link_rewrite):$product->link_rewrite;
                if (empty($name)) {
                    $name = Tools::link_rewrite(
                        (is_array($product->name))?$this->getFNE($product->name):$product->name
                    );
                }
            }
            $sql = "SELECT * FROM "._DB_PREFIX_."sirv_images
            WHERE item_id = '".pSQL($ids)."' AND type = '".pSQL($itemType)."'";
        } elseif ($itemType == 'CATEGORY') {
            $ids = (int)$ids;
            $category_id = $ids;
            if ($category_id) {
                $category = new Category($category_id, false);
                $name = (is_array($category->link_rewrite))?
                $this->getFNE($category->link_rewrite):
                $category->link_rewrite;
            }
            $sql = "SELECT * FROM "._DB_PREFIX_."sirv_images
            WHERE item_id = '".pSQL($ids)."' AND type = '".pSQL($itemType)."'";
        } elseif ($itemType == 'CUSTOM') {
            $sql = "SELECT * FROM "._DB_PREFIX_."sirv_images
            WHERE path = '".pSQL($imgPathShort)."' AND type = 'CUSTOM'";
        } elseif ($itemType == 'MAGIC360')  {
            $sql = "SELECT * FROM "._DB_PREFIX_."sirv_images
            WHERE item_id = '".pSQL($ids)."' AND type = '".pSQL($itemType)."'";
        } else {
            $sql = "SELECT * FROM "._DB_PREFIX_."sirv_images
            WHERE item_id = '".pSQL(Tools::substr(md5($imgPathShort), 0, 10))."' AND type = '".pSQL($itemType)."'";
        }

        $image = Db::getInstance()->getRow($sql);

        $format_filename = preg_replace('/(.*)\/[^\/]*$/ims', '$1/fileType', $imgPath);
        $format = (file_exists($format_filename))?Tools::file_get_contents($format_filename):'';
        if (empty($format)) {
            $format = 'jpg';
        }

        if (empty($name)) {
            $name = preg_replace('/.*\/(.*)\.[a-z]{1,}/ims', '$1', $imgLink);
        }
        

        if ($itemType=='PRODUCT') {
            $imgSirvPath = str_replace(
                _PS_ROOT_DIR_.'/',
                '',
                _PS_PROD_IMG_DIR_.Image::getImgFolderStatic($ids).$name.".".$format
            );
        } elseif ($itemType=='CATEGORY') {
            $imgSirvPath = str_replace(
                _PS_ROOT_DIR_.'/',
                '',
                _PS_CAT_IMG_DIR_.$ids.'/'.$name.".".$format
            );
        } else {
            $imgSirvPath = str_replace(_PS_ROOT_DIR_.'/', '', $imgPath);
        }


        if (empty($imgPath)) {
            return false;
        }

        // if (empty($image)) {
        //     return false;
        // }



        if (!file_exists($imgPath) && !file_exists(_PS_ROOT_DIR_.'/'.$imgPath)) {
            if ($image && $image['id'] && $image['status'] == 'FAILED') {
                return false;
            }

            if ($image && $image['id']) {
                Db::getInstance()->update('sirv_images', array(
                    'path'      => pSQL($imgPathShort),
                    'msg'       => 'File does not exist',
                    'status'    => pSQL('FAILED'),
                ), 'id = '.(int)$image['id']);
            } else {
                Db::getInstance()->insert('sirv_images', array(
                    'item_id'           => ($itemType=='CUSTOM')?Tools::substr(md5($imgPathShort), 0, 10):(int)$ids,
                    'path'              => pSQL($imgPathShort),
                    'msg'               => 'File does not exist',
                    'status'            => pSQL('FAILED'),
                    'type'              => $itemType
                ), true, false, Db::REPLACE);
            }
            return false;
        }

        $imageURL = _PS_BASE_URL_.__PS_BASE_URI__.$imgPathShort;

        if (isset($_GET['magic'])) {
            print_r($image);
            print "@";
        }

        if ($this->isFileExcluded($imageURL)) {
            Db::getInstance()->update('sirv_images', array(
                'msg'       => 'Excluded',
                'status'    => pSQL('FAILED'),
            
            ), 'id = '.
            (int)$image['id']);
            return false;
        }

        if ($this->isPageExcluded()) {
            return false;
        }

        if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') {
            $imageURL = str_replace('http:', 'https:', $imageURL);
        }

        if (!$image) {
            $filesize = (file_exists($imgPath))?filesize($imgPath):@filesize(_PS_ROOT_DIR_.'/'.$imgPath);
            $ftime = (file_exists($imgPath))?filemtime($imgPath):@filemtime(_PS_ROOT_DIR_.'/'.$imgPath);
            if ($filesize) {
                if ($itemType=='CUSTOM') {
                    $image = Db::getInstance()->getRow(
                        "SELECT * FROM "._DB_PREFIX_."sirv_images WHERE path = '". pSQL($imgPathShort)."'"
                    );
                }
                if (!empty($image) && !empty($image['id'])) {
                    $image_id = $image['id'];
                } else {
                    Db::getInstance()->insert('sirv_images', array(
                        'item_id'           => ($itemType=='CUSTOM')?
                        Tools::substr(md5($imgPathShort), 0, 10):
                        (int)$ids,
                        'path'              => pSQL($imgPathShort),
                        'size'              => (int)$filesize,
                        'timestamp'         => pSQL(date('Y-m-d H:i:s', $ftime)),
                        'timestamp_synced'  => pSQL(date('Y-m-d H:i:s', $ftime)),
                        'status'            => pSQL('NEW'),
                        'type'              => $itemType
                    ), true, false, Db::REPLACE);
                    $image_id = Db::getInstance()->Insert_ID();
                    $image = Db::getInstance()->getRow(
                        "SELECT * FROM "._DB_PREFIX_."sirv_images WHERE id = ". (int)$image_id
                    );
                }
            }
        }

        if (empty($image_id) && !empty($image) && !empty($image['id'])) {
            $image_id = $image['id'];
        }

        if ($image_id) {
            if (empty($GLOBALS['syncedsirvimages'])) {
                $GLOBALS['syncedsirvimages'] = array();
            }
            
            if (empty($GLOBALS['syncedsirvimages'][$image_id])) {
                $GLOBALS['syncedsirvimages'][$image_id] = 0;
            }
        }

        if ($image && strtotime($image['timestamp'])<@filemtime($imgPath)||$image['size']!=@filesize($imgPath)) {
            $ftime = (file_exists($imgPath))?filemtime($imgPath):@filemtime(_PS_ROOT_DIR_.'/'.$imgPath);
            $this->APIClient = $this->getAPIClient();
            $this->APIClient->deleteFile(Configuration::get('SIRV_FOLDER').'/'.$imgSirvPath);
            Db::getInstance()->update('sirv_images', array(
                'timestamp'         => pSQL(date('Y-m-d H:i:s', $ftime)),
                'timestamp_synced'  => pSQL(date('Y-m-d H:i:s', $ftime)),
                'sirvpath'          => pSQL(''),
                'status'            => pSQL('NEW'),
            ), 'id = '.(int)$image['id']);
            $image = Db::getInstance()->getRow(
                "SELECT * FROM "._DB_PREFIX_."sirv_images WHERE id = ". (int)$image['id']
            );
            //return false;
        }

        if ($image && $image['status'] == 'NEW') {
            $filesize = filesize($imgPath);

            $filesize = (file_exists($imgPath))?filesize($imgPath):@filesize(_PS_ROOT_DIR_.'/'.$imgPath);
            $ftime = (file_exists($imgPath))?filemtime($imgPath):@filemtime(_PS_ROOT_DIR_.'/'.$imgPath);

            Db::getInstance()->update('sirv_images', array(
                'path'              => pSQL($imgPathShort),
                'size'              => (int)$filesize,
                'timestamp'         => pSQL(date('Y-m-d H:i:s', $ftime)),
                'timestamp_synced'  => pSQL(date('Y-m-d H:i:s', $ftime)),
            ), 'id = '.(int)$image['id']);
            $image = Db::getInstance()->getRow(
                "SELECT * FROM "._DB_PREFIX_."sirv_images WHERE id = ". (int)$image['id']
            );

            try {
                $sirvImageRelativeURL = Configuration::get('SIRV_FOLDER').'/'.$imgSirvPath;

                if (false && !empty($image['sirvpath']) &&
                    strtotime($image['timestamp'])>=filemtime($imgPath) &&
                    $image['size']==filesize($imgPath) && $this->checkIfImageExists($sirvImageRelativeURL)) {
                    Db::getInstance()->update('sirv_images', array(
                        'status'            => pSQL('SYNCED')
                    ), 'id = '.(int)$image['id']);
                } else {
                    $size = filesize($imgPath);
                    
                    if ($itemType == 'PRODUCT') {
                        $imageType = 'products';
                    } elseif ($itemType == 'CATEGORY') {
                        $imageType = 'category';
                    } elseif ($itemType == 'MAGIC360') {
                        $imageType = 'magic360';
                    } elseif ($itemType == 'MAGICSCROLL') {
                        $imageType = 'magicscroll';
                    } elseif ($itemType == 'MAGICSLIDESHOW') {
                        $imageType = 'magicslideshow';
                    } elseif ($itemType == 'CUSTOM') {
                        $imageType = 'custom';
                    }

                    $file = $this->uploadFile(
                        $imgPath,
                        preg_replace( '/\/{1,}/ims', '/', '/'.$sirvImageRelativeURL ),
                        $imageURL,
                        array(
                            'id'            => $image['id'],
                            'size'          => $size,
                            'imgSirvPath'   => (empty($image['sirvpath']))?$imgSirvPath:$image['sirvpath'],
                            'imgPath'       => $imgPath,
                            'imageType'     => $imageType
                        ),
                        $wait
                    );

                    if ($file) {
                        Db::getInstance()->update('sirv_images', array(
                            'size'              => (int)$size,
                            'sirvpath'          => pSQL((empty($image['sirvpath']))?$imgSirvPath:$image['sirvpath']),
                            'timestamp'         => pSQL(date('Y-m-d H:i:s', $ftime)),
                            'timestamp_synced'  => pSQL(date('Y-m-d H:i:s', $ftime)),
                            'status'            => pSQL('SYNCED'),
                        ), 'id = '.(int)$image['id']);
                        return $imgLink = $this->getSirvURL().Configuration::get('SIRV_FOLDER').'/'.$imgSirvPath;
                    } else {
                        return false;
                    }
                }
            } catch (Exception $e) {
                return false;
            }

            $image_id = $image['id'];
            $image = Db::getInstance()->getRow("SELECT * FROM "._DB_PREFIX_."sirv_images WHERE id = ". (int)$image_id);

            $GLOBALS['syncedsirvimages'][$image_id]++;

            return false;
        }



        if ($image && $image['status'] == 'PROCESSING' && $GLOBALS['syncedsirvimages'][$image['id']] == 0) {
            $GLOBALS['syncedsirvimages'][$image_id]++;
            $sirvImageRelativeURL = Configuration::get('SIRV_FOLDER').'/'.$imgSirvPath;
            if ($this->checkIfImageExists($sirvImageRelativeURL)) {
                $ftime = (file_exists($imgPath))?filemtime($imgPath):@filemtime(_PS_ROOT_DIR_.'/'.$imgPath);

                Db::getInstance()->execute(
                    'UPDATE `'._DB_PREFIX_.'sirv_images` SET 
                    timestamp = \''.pSQL(date('Y-m-d H:i:s', $ftime)).'\', 
                    status = \'SYNCED\', 
                    timestamp_synced = \''.pSQL(date('Y-m-d H:i:s', $ftime)).'\' 
                    WHERE id = '.(int)$image['id']
                );

                if (Configuration::get('SIRV_DELETE_THUMBNAILS')=='Delete') {
                    if ($itemType == 'PRODUCT') {
                        $imageType = 'products';
                    } elseif ($itemType == 'CATEGORY') {
                        $imageType = 'category';
                    } elseif ($itemType == 'MAGIC360') {
                        $imageType = 'magic360';
                    } elseif ($itemType == 'MAGICSCROLL') {
                        $imageType = 'magicscroll';
                    } elseif ($itemType == 'MAGICSLIDESHOW') {
                        $imageType = 'magicslideshow';
                    } elseif ($itemType == 'CUSTOM') {
                        $imageType = '';
                    }

                    if (!empty($imageType)) {
                        foreach ($this->ImageTypes as $type_name => $type) {
                            $newFilePath = preg_replace('/(.*)(\.[a-z]{1,})$/ims', '$1-'.$type_name.'$2', $imgPath);
                            //if ($imageType == 'product' && $type[$imageType] == 1) {
                            @unlink($newFilePath);
                            $newFilePath = preg_replace('/(.*)(\.[a-z]{1,})$/ims', '$1-'.$type_name.'2x'.'$2', $imgPath);
                            //if ($imageType == 'product' && $type[$imageType] == 1) {
                            @unlink($newFilePath);
                        }
                    }
                }
                $image_id = $image['id'];
                $image = Db::getInstance()->getRow("SELECT * FROM "._DB_PREFIX_."sirv_images
                    WHERE id = ". (int)$image_id);
            } else {
                Db::getInstance()->update('sirv_images', array(
                    'checks'           => $image['checks']+1
                ), 'id = '.(int)$image['id']);
                if ($image['checks']+1 > 5) {
                    Db::getInstance()->update('sirv_images', array(
                        'status'           => pSQL('FAILED'),
                        'msg'           => '5 failed sync attemps',
                        'checks'           => 0,
                    ), 'id = '.(int)$image['id']);
                    $image['status'] = 'FAILED';
                }
                return false;
            }
        }

        // if ($image && strtotime($image['timestamp'])<@filemtime($imgPath)||$image['size']!=@filesize($imgPath)) {
        //     $ftime = (file_exists($imgPath))?filemtime($imgPath):@filemtime(_PS_ROOT_DIR_.'/'.$imgPath);
        //     Db::getInstance()->update('sirv_images', array(
        //         'timestamp'         => pSQL(date('Y-m-d H:i:s', $ftime)),
        //         'timestamp_synced'  => pSQL(date('Y-m-d H:i:s', $ftime)),
        //         'sirvpath'          => pSQL(''),
        //         'status'            => pSQL('NEW'),
        //     ), 'id = '.(int)$image['id']);
        //     return false;
        // }

        if ($image['status'] == 'SYNCED') {
            $imgLink = $this->getSirvURL().( Configuration::get('SIRV_FOLDER')==''?'':Configuration::get('SIRV_FOLDER').'/' ).$image['sirvpath'];
        } else {
            $imgLink = false;
        }

        

        $GLOBALS['syncedImagesCache'][$cacheID] = $imgLink;

        return $imgLink;
    }

    public function ajaxProcessSirvFillTable()
    {

        $sql = 'INSERT IGNORE INTO '._DB_PREFIX_.'sirv_images (item_id,path,size,timestamp,timestamp_synced,status,type)
        SELECT id_image,\'\',0,NULL,NULL,\'NEW\',\'PRODUCT\' FROM '._DB_PREFIX_.'image 
        WHERE NOT(id_product IS NULL)';
        Db::getInstance()->execute($sql);

        $sql = 'SELECT * FROM `'._DB_PREFIX_.'category`';
        $result = Db::getInstance()->ExecuteS($sql);
        foreach ($result as $row) {
            if (@file_exists(_PS_CAT_IMG_DIR_.$row['id_category'].'.jpg')) {
                $sql = 'INSERT IGNORE INTO '._DB_PREFIX_.'sirv_images'.
                ' (item_id,path,size,timestamp,timestamp_synced,status,type)'.
                'VALUES('.(int)$row['id_category'].',\'\',0,NULL,NULL,\'NEW\',\'CATEGORY\')';
                Db::getInstance()->execute($sql);
            }
        }



        $sql_360 = 'SHOW TABLES LIKE "%'._DB_PREFIX_.'magic360_images"';
        $row_360 = Db::getInstance()->ExecuteS($sql_360);
        if ($row_360) {
            $sql = 'INSERT IGNORE INTO '._DB_PREFIX_.'sirv_images
            (item_id,path,size,timestamp,timestamp_synced,status,type)
            SELECT id_image,\'\',0,NULL,NULL,\'NEW\',\'MAGIC360\' FROM '.
            _DB_PREFIX_.'magic360_images';
            Db::getInstance()->execute($sql);
        }

        $sql_scroll = 'SHOW TABLES LIKE "%'._DB_PREFIX_.'magicscroll_images"';
        $row_scroll = Db::getInstance()->ExecuteS($sql_scroll);
        if ($row_scroll) {
            $sql = 'INSERT IGNORE INTO '._DB_PREFIX_.'sirv_images
            (item_id,path,size,timestamp,timestamp_synced,status,type)
            SELECT id,\'\',0,NULL,NULL,\'NEW\',\'MAGICSCROLL\' FROM '.
            _DB_PREFIX_.'magicscroll_images';
            Db::getInstance()->execute($sql);
        }

        $sql_slideshow = 'SHOW TABLES LIKE "%'._DB_PREFIX_.'magicslideshow_images"';
        $row_slideshow = Db::getInstance()->ExecuteS($sql_slideshow);
        if ($row_slideshow) {
            $sql = 'INSERT IGNORE INTO '._DB_PREFIX_.'sirv_images
            (item_id,path,size,timestamp,timestamp_synced,status,type)
            SELECT id,\'\',0,NULL,NULL,\'NEW\',\'MAGICSLIDESHOW\' FROM '.
            _DB_PREFIX_.'magicslideshow_images';
            Db::getInstance()->execute($sql);
        }
    }

    public function ajaxProcessSirvRefreshStorageInfo($doOutput = true)
    {

        $this->APIClient = $this->getAPIClient();

        $SirvStatus = $this->APIClient->preOperationCheck();

        if ($SirvStatus) {
            $storageInfo = $this->getStorageInfo(true);
        }
        
        $storageInfo['user'] = Configuration::get('SIRV_ACCOUNT_EMAIL');
        $this->smarty->assign('storageInfo', $storageInfo);
        $sirv_stat_last_update = (array)json_decode(Configuration::get('SIRV_STAT'), true);
        $this->smarty->assign(
            'sirv_stat_last_update',
            date("Y-m-d H:i:s", $sirv_stat_last_update['time'])
        );

        $this->smarty->assign('storageInfoShowAPI', Tools::getValue('storageInfoShowAPI') == 1);

        if ($doOutput) {
            echo $this->display(__FILE__, 'views/templates/admin/stats-block.tpl');
            die();
        }
    }

    public function ajaxProcessSirvEmptyViewsCache()
    {

        if (Tools::getValue('method') == 'ALL') {
            Db::getInstance()->delete(
                'sirv_views_cache'
            );
        } elseif (Tools::getValue('method') == 'EMPTY' || Tools::getValue('method') == 'FAILED') {
            Db::getInstance()->delete(
                'sirv_views_cache',
                'status IN ("EMPTY", "FAILED")'
            );
        }

        $sql = "SELECT COUNT(*) FROM "._DB_PREFIX_."sirv_views_cache";
        $viewsCacheAll = Db::getInstance()->getValue($sql);

        $sql = "SELECT COUNT(*) FROM "._DB_PREFIX_."sirv_views_cache
        WHERE status = 'FAILED'";
        $viewsCacheMissed = Db::getInstance()->getValue($sql);

        echo json_encode(array('viewsCacheAll'=>$viewsCacheAll, 'viewsCacheMissed'=>$viewsCacheMissed));
        die();
    }

    public function getFNE($arr)
    {
        foreach ($arr as $value) {
            if (!empty($value)) {
                return $value;
            }
        }
    }


    public function ajaxProcessSirvSyncImages()
    {

        $this->APIClient = $this->getAPIClient();

        $sql = 'SELECT * FROM '._DB_PREFIX_.'sirv_images 
        WHERE status != "FAILED"  AND status != "SYNCED"'.
        ' ORDER BY IF(status="NEW",0,1), IF(status="PROCESSING", checks , 10) LIMIT 10';
        $results = Db::getInstance()->ExecuteS($sql);

        if (empty($results) || count($results) == 0) {
            $this->ajaxProcessSirvFillTable();
            $results = Db::getInstance()->ExecuteS($sql);
        }

        ini_set('max_execution_time', 40);

        $maxExecutionTime = (int)ini_get('max_execution_time');

        if ($maxExecutionTime==0) {
            $maxExecutionTime = 10;
        }

        $startTime = time();

        if ($results) {
            foreach ($results as $row) {
                $product_link_rewrite = '';
                if ($row['type']=='PRODUCT') {
                    $sql = 'SELECT id_product FROM '._DB_PREFIX_.'image WHERE id_image = '.(int)$row['item_id'];
                    $row_product = Db::getInstance()->getRow($sql);
                    $id_product = $row_product['id_product'];
                    $product = new Product($id_product, false);
                    if (is_array($product->link_rewrite)) {
                        $product->link_rewrite = $this->getFNE($product->link_rewrite);
                    }
                    $imgPath = _PS_PROD_IMG_DIR_.Image::getImgFolderStatic($row['item_id']).$row['item_id'].'.jpg';
                    $product_link_rewrite = $product->link_rewrite;
                    if (empty($product_link_rewrite)) {
                        $product_link_rewrite = Tools::link_rewrite(
                            (is_array($product->name))?$this->getFNE($product->name):$product->name
                        );
                    }
                }

                if ($row['type']=='CATEGORY') {
                    $id_category = $row['item_id'];
                    $category = new Category($id_category, false);
                    if (is_array($category->link_rewrite)) {
                        $category->link_rewrite = reset($category->link_rewrite);
                    }
                    $imgPath = _PS_CAT_IMG_DIR_.$row['item_id'].'.jpg';
                    $product_link_rewrite = $product->link_rewrite;
                }

                if ($row['type']=='MAGIC360') {
                    $sql = 'SELECT * FROM '._DB_PREFIX_.'magic360_images
                    WHERE id_image = '.(int)$row['item_id'];
                    $row_product = Db::getInstance()->getRow($sql);
                    $id_product = $row_product['id_product'];
                    $ids = $id_product.'-'.$row_product['id_image'];
                    $imgPath = _PS_IMG_DIR_.'magic360/'.$ids.'.jpg';
                    $product_link_rewrite = $ids;
                }
                if ($row['type']=='MAGICSCROLL') {
                    $sql = 'SELECT * FROM '._DB_PREFIX_.'magicscroll_images
                    WHERE id = '.(int)$row['item_id'];
                    $row_image = Db::getInstance()->getRow($sql);
                    $imgPath = _PS_IMG_DIR_.'magicscroll/'.$row_image['name'].'.'.$row_image['ext'];
                    $product_link_rewrite = $row_image['name'];
                }
                if ($row['type']=='MAGICSLIDESHOW') {
                    $sql = 'SELECT * FROM '._DB_PREFIX_.'magicslideshow_images
                    WHERE id = '.(int)$row['item_id'];
                    $row_image = Db::getInstance()->getRow($sql);
                    $imgPath = _PS_IMG_DIR_.'magicslideshow/'.$row_image['name'].'.'.$row_image['ext'];
                    $product_link_rewrite = $row_image['name'];
                }
                if ($row['type']=='CUSTOM') {
                    $product_link_rewrite = false;
                    $imgPath = _PS_ROOT_DIR_.'/'.$row['path'];
                }

                $this->syncImage(
                    $product_link_rewrite,
                    $row['item_id'],
                    '',
                    '',
                    $imgPath,
                    $row['type'],
                    true
                );

                if ($maxExecutionTime && (time() - $startTime > $maxExecutionTime - 1)) {
                    break;
                }
            }
        }

        $this->processFetchQueue();

        $cacheInfo = $this->getCacheInfo();

        $cacheInfo['isMuted'] = $this->APIClient->isMuted()?'1':'0';

        echo json_encode($cacheInfo);

        die();
    }

    public function ajaxProcessSirvAnalyzeImages()
    {

        $data = Tools::getValue('data');

        if (empty($data['total_ps_images'])) {
            $sql = 'SELECT COUNT(*) FROM '._DB_PREFIX_.'image WHERE NOT(id_product IS NULL)';
            $data['total_ps_images'] = Db::getInstance()->getValue($sql);
        }

        $sql = 'SELECT i.* , s.id as sirv_image_id FROM '._DB_PREFIX_.'image i
        LEFT JOIN '._DB_PREFIX_.'sirv_images s ON s.item_id = i.id_image AND s.status = "SYNCED" AND s.type = "PRODUCT"
        WHERE NOT(id_product IS NULL)'.
        ( ($data['delete'])?' HAVING NOT(sirv_image_id IS NULL) ':'' ).
        'LIMIT '.$data['offset'].','.$data['step'];

        $sql = 'SELECT i.* , s.id as sirv_image_id 
        FROM '._DB_PREFIX_.'sirv_images s 
        LEFT JOIN '._DB_PREFIX_.'image i ON s.item_id = i.id_image AND NOT(id_product IS NULL)
        WHERE s.status = "SYNCED" AND s.type = "PRODUCT"  '.
        ( ($data['delete'])?' HAVING NOT(sirv_image_id IS NULL) ':'' ).
        'LIMIT '.$data['offset'].','.$data['step'];
        
        $results = Db::getInstance()->ExecuteS($sql);

        if (empty($results) || count($results) == 0) {
            //die();
        }

        ini_set('max_execution_time', 40);

        $maxExecutionTime = (int)ini_get('max_execution_time');

        if ($maxExecutionTime==0) {
            $maxExecutionTime = 10;
        }

        $startTime = time();

        if ($results) {
            foreach ($results as $row) {
                foreach ($this->ImageTypes as $type_name => $type) {
                    foreach(['jpg', 'webp'] as $extension) {
                        foreach(['', '2x'] as $x2) {
                            $imgPath = _PS_PROD_IMG_DIR_.
                            Image::getImgFolderStatic($row['id_image']).$row['id_image'].'-'.$type_name.$x2.'.'.$extension;
                            if (file_exists($imgPath)) {
                                if ($data['delete']) {
                                    $data['deleted_size'] += filesize($imgPath);
                                    @unlink($imgPath);
                                } else {
                                    $data['total_images']++;
                                    $data['total_size'] += filesize($imgPath);
                                    if (!empty($row['sirv_image_id'])) {
                                        $data['delete_images']++;
                                        $data['delete_size'] += filesize($imgPath);
                                    }
                                }
                            }
                        }
                    }

                    unset($type);
                }

                if (!empty($row['sirv_image_id']) && empty($data['delete'])) {
                    $data['total_ps_images_synced']++;
                }

                if ($maxExecutionTime && (time() - $startTime > $maxExecutionTime - 1)) {
                    break;
                }
            }
        }

        $data['total_size_text'] = $this->getFormatedFileSize($data['total_size'], 0);
        $data['delete_size_text'] = $this->getFormatedFileSize($data['delete_size'], 0);
        $data['deleted_size_text'] = $this->getFormatedFileSize($data['deleted_size'], 0);

        echo json_encode($data);

        die();
    }

    public function ajaxProcessSirvCleanupImages()
    {

        $data = Tools::getValue('data');

        $this->APIClient = $this->getAPIClient();

        if (empty($data['total_ps_images'])) {
            
            // $sql = 'SELECT (SELECT id_image FROM '._DB_PREFIX_.'image pi WHERE pi.id_image = si.item_id) id_image, si.* 
            // FROM '._DB_PREFIX_.'sirv_images si WHERE type = "PRODUCT" AND status = "SYNCED"
            // HAVING ISNULL(id_image)';
            // $results = Db::getInstance()->ExecuteS($sql);

            if (empty($_SESSION['delete-files-result'])) {
                $_SESSION['delete-files-result'] = [];
            }


            $wildcards = ['/', '*'];
            $wildcards_replace = ['\\/', '*'];

            $q = [];
            
            // $cdateTo = strtotime(trim( date('Y-m-d', strtotime('-90 days')) ));
            // $q[] = 'ctime : [ * TO '.( date("Y-m-d", $cdateTo) ).']';

            // $adateTo = strtotime(trim( date('Y-m-d', strtotime('-60 days')) ));
            // $q[] = 'atime : [ * TO '.( date("Y-m-d", $adateTo) ).']';
    
            $folders = ['/js', '/modules', '/themes', '/'.Configuration::get('SIRV_FOLDER')];

            $qf = [];
            foreach($folders as $folder) {
                $folder = preg_replace('/\/{1,}/ims', '/',  $folder);
                $qf[] = 'dirname.paths:"'.$folder.'"';
            }

            $q[] = '('.implode(' OR ', $qf).')';

            $q[] = '(contentType:"image/*" OR contentType:"text/*" OR contentType:"application/javascript/*")';

            $_SESSION['mass-delete-files_query'] = str_replace($wildcards, $wildcards_replace,  implode(' AND ', $q));
            $result = $this->APIClient->getSearchResults($_SESSION['mass-delete-files_query'], 10000, 0, false);

            $list = $result['results'];

            foreach($list as $i => $file) {
                if ($file->_source->isDirectory) {
                    unset($list[$i]);
                }
            }

            $_SESSION['mass-delete-files_list'] = $list;

            if (empty($data)) {
                $data = [];
            }

            $data['total_ps_images'] = $data['total_ps_images_synced'] = $data['delete_images'] = count($list);
            $data['info'] = 1;
            foreach ($list as $row) {
                $data['delete_size'] += $row->_source->size;
            }
        } else {
            $data['info'] = 0;

            $this->APIClient = $this->getAPIClient();

            ini_set('max_execution_time', 40);

            $maxExecutionTime = (int)ini_get('max_execution_time');

            if ($maxExecutionTime==0) {
                $maxExecutionTime = 10;
            }

            $startTime = time();

            if ($_SESSION['mass-delete-files_list']) {
                foreach ($_SESSION['mass-delete-files_list'] as $i => $row) {

                    $sql = "SELECT * FROM "._DB_PREFIX_."sirv_images WHERE path = '".pSQL( str_replace('/'.Configuration::get('SIRV_FOLDER').'/', '', $row->_source->filename) )."'";

                    $image = Db::getInstance()->getRow($sql);

                    if (!empty($image['id'])) {
                        Db::getInstance()->delete(
                            'sirv_images',
                            'id = '.intval($image['id'])
                        );
                    }

                    $data['deleted_size'] += $row->_source->size;
                    $data['delete'] = 1;

                    $res = $this->APIClient->deleteFile($row->_source->filename);
                    
                    unset($_SESSION['mass-delete-files_list'][$i]);

                    if ($maxExecutionTime && (time() - $startTime > $maxExecutionTime - 1)) {
                        break;
                    }
                }
            }            
        }


        //$data['total_size_text'] = $this->getFormatedFileSize($data['total_size'], 0);
        $data['delete_size_text'] = $this->getFormatedFileSize($data['delete_size'], 0);
        $data['deleted_size_text'] = $this->getFormatedFileSize($data['deleted_size'], 0);



        echo json_encode($data);

        die();
    }



    public function ajaxProcessSirvEstimateImages()
    {

        $data = Tools::getValue('data');

        if (empty($data['total_ps_images'])) {
            $sql = 'SELECT COUNT(*) FROM '._DB_PREFIX_.'image WHERE NOT(id_product IS NULL)';
            $data['total_ps_images'] = Db::getInstance()->getValue($sql);
        }

        $sql = 'SELECT i.* FROM '._DB_PREFIX_.'image i
        WHERE NOT(id_product IS NULL)'.
        'LIMIT '.$data['offset'].','.$data['step'];
        $results = Db::getInstance()->ExecuteS($sql);

        if (empty($results) || count($results) == 0) {
            //die();
        }

        ini_set('max_execution_time', 40);

        $maxExecutionTime = (int)ini_get('max_execution_time');

        if ($maxExecutionTime==0) {
            $maxExecutionTime = 10;
        }

        $startTime = time();

        if ($results) {
            foreach ($results as $row) {
                foreach(['jpg', 'webp'] as $extension) {
                    $imgPath = _PS_PROD_IMG_DIR_.
                    Image::getImgFolderStatic($row['id_image']).$row['id_image'].'.'.$extension;
                    if (file_exists($imgPath)) {
                        $data['total_images']++;
                        $data['total_size'] += filesize($imgPath);
                    }
                }

                if ($maxExecutionTime && (time() - $startTime > $maxExecutionTime - 1)) {
                    break;
                }
            }
        }

        $data['total_size_text'] = $this->getFormatedFileSize($data['total_size'], 0);
        $data['delete_size_text'] = $this->getFormatedFileSize(@$data['delete_size'], 0);
        $data['deleted_size_text'] = $this->getFormatedFileSize(@$data['deleted_size'], 0);

        if ($data['offset'] >= $data['total_ps_images']) {
            Configuration::updateValue('SIRV_DATA_ESTIMATED_JSON', json_encode($data));
            Configuration::updateValue('SIRV_DATA_ESTIMATED', 1);
        }

        echo json_encode($data);

        die();
    }

    public function ajaxProcessSirvGetFailedImagesList()
    {
        $sql = 'SELECT * FROM '._DB_PREFIX_.'sirv_images WHERE status = "FAILED"';
        $results = Db::getInstance()->ExecuteS($sql);

        $list = array();
        if ($results) {
            foreach ($results as $row) {
                $list[] = array( 'p' => $row['path'], 'm' => $row['msg']);
            }
        }
        
        echo json_encode($list);

        die();
    }

    public function ajaxProcessSirvGetFailedImagesListDownload()
    {
        $sql = 'SELECT path, msg FROM '._DB_PREFIX_.'sirv_images WHERE status = "FAILED"';
        $results = Db::getInstance()->ExecuteS($sql);

        ob_start();
        $df = fopen("php://output", 'w');
        fputcsv($df, array('File path', 'File URL', 'Error'));

        if ($results) {
            foreach ($results as $row) {
                $row = array(
                    'path' => $row['path'],
                    'url' => _PS_BASE_URL_.__PS_BASE_URI__.$row['path'],
                    'msg' => $row['msg'],
                );
                fputcsv($df, $row);
            }
        }
        fclose($df);
        $content = ob_get_clean();
        
        header("Content-Type: text/csv");
        header("Content-Disposition: attachment; filename=\"sirv-failed-images.csv\"");
        header("Cache-Control: no-store, no-cache, must-revalidate");
        header("Cache-Control: post-check=0, pre-check=0", false);
        //if (array_key_exists('HTTPS', $_SERVER) && $_SERVER['HTTPS'] == "on") {
        if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on") {            
            header("Pragma: public");
        } else {
            header("Pragma: no-cache");
        }
        header("Cache-Control: private");
        header("Expires: 0");

        echo $content;


        die();
    }

    public function ajaxProcessSirvGetFailedImagesListHTML()
    {


        $sql = 'SELECT path, msg FROM '._DB_PREFIX_.'sirv_images WHERE status = "FAILED"';
        $results = Db::getInstance()->ExecuteS($sql);

        header("Content-Type: text/html");
        $GLOBALS['results'] = $results;
        require('templates/failed-images.tpl.php');
        unset($GLOBALS['results']);
        die();
    }

    public function ajaxProcessSirvDismissReviewBlock()
    {
        Configuration::updateValue('SIRV_INSTALLED_DATE', time() + 60*60*24*30*12*10);
        die();
    }
    
    public function displayForm()
    {
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $fields_form = array();
        $i = 0;
        $oldParamGroup = '';
        foreach ($this->params as $param_id => $param) {
            if (!empty($param['hidden'])) {
                continue;
            }
            if ($param['group']!=$oldParamGroup) {
                if ($oldParamGroup!='') {
                    $fields_form[$i]['form']['submit'] = array(
                        'title' => $this->l('Save settings'),
                        'class' => 'btn btn-default pull-right'
                    );
                    $i++;
                }
                $fields_form[$i]['form']['legend'] = array(
                    'title' => $param['group']
                );
                if (!empty($param['group_description'])) {
                    $fields_form[$i]['form']['description'] = $param['group_description'];
                }
                $oldParamGroup = $param['group'];
                $fields_form[$i]['form']['input'] = array();
            }

            if ($param_id == 'profile') {
                $param['values'] = $this->getProfilesList();
            }

            if ($param['type']=='array') {
                $values = array();
                foreach ($param['values'] as $j => $value) {
                    $values[] = array(
                        'id'    => $param_id.'_'.$j,
                        'value' => ($param['subType'] == 'checkbox') ?
                        str_replace('Yes', '1', str_replace('No', '0', $value)) : $value,
                        'label' => (isset($param['valuesLabels'][$j]))?$param['valuesLabels'][$j]:$value
                    );
                }
                if ($param['subType'] == 'radio') {
                    $fields_form[$i]['form']['input'][] = array(
                        'type' => 'radio',
                        'label' => $param['label'],
                        'desc' => (!empty($param['description']))?$param['description']:'',
                        'name' => 'SIRV_'.Tools::strtoupper($param_id),
                        'values' => $values,
                        'size' => 20,
                        'required' => false
                    );
                } elseif ($param['subType'] == 'checkbox') {
                    if ($param_id == 'sirvjscomponents') {
                        $values = array();
                        $values[] = array(
                            'id_option' => 'ALL',
                            //'name' => ucwords($value),
                            'name' => $this->l('Select all'),
                            'val' => 'ALL',
                        );
                        foreach ($param['values'] as $j => $value) {
                            $values[] = array(
                                'id_option' => $value,
                                //'name' => ucwords($value),
                                'name' => (isset($param['valuesLabels'][$j]))?$param['valuesLabels'][$j]:$value,
                                'val' => $value,
                            );
                        }

                        $field = array(
                            'type' => 'checkbox',
                            'label' => $param['label'],
                            'is_bool' => false,
                            'desc' => (!empty($param['description']))?$param['description']:'',
                            'name' => 'SIRV_'.Tools::strtoupper($param_id),
                            'values' => array(
                                'query' => $values,
                                'id' => 'id_option',
                                'name' => 'name'
                            ),
                            
                            'size' => 20,
                            'required' => false
                        );

                        $fields_form[$i]['form']['input'][] = $field;
                    } else {
                        $fields_form[$i]['form']['input'][] = array(
                            'type' => 'switch',
                            'label' => $param['label'],
                            'is_bool' => true,
                            'desc' => (!empty($param['description']))?$param['description']:'',
                            'name' => 'SIRV_'.Tools::strtoupper($param_id),
                            'values' => $values,
                            'size' => 20,
                            'required' => false
                        );
                    }
                } elseif ($param['subType'] == 'select') {
                    $values = array();
                    if ($param_id == 'profile') {
                        $values[] = array(
                            'id_option'    => '',
                            'name' => $this->l('No profile'),
                        );
                    }
                    foreach ($param['values'] as $j => $value) {
                        $values[] = array(
                            'id_option'    => $value,
                            'name' => (isset($param['valuesLabels'][$j]))?$param['valuesLabels'][$j]:$value,
                        );
                    }
                    
                    
                    $fields_form[$i]['form']['input'][] = array(
                        'type' => 'select',
                        'label' => $param['label'],
                        'is_bool' => false,
                        'desc' => (!empty($param['description']))?$param['description']:'',
                        'name' => 'SIRV_'.Tools::strtoupper($param_id),
                        'options' => array(
                            'query' => $values,
                            'id' => 'id_option',
                            'name' => 'name'
                        ),
                        'size' => 1,
                        'required' => false
                    );
                }
            } elseif ($param['type']=='text') {
                $fields_form[$i]['form']['input'][] = array(
                    'type' => 'text',
                    'label' => $param['label'],
                    'desc' => $param['description'],
                    'name' => 'SIRV_'.Tools::strtoupper($param_id),
                    'size' => 20,
                    'required' => false
                );
            } elseif ($param['type']=='textarea') {
                $fields_form[$i]['form']['input'][] = array(
                    'type' => 'textarea',
                    'label' => $param['label'],
                    'desc' => $param['description'],
                    'name' => 'SIRV_'.Tools::strtoupper($param_id),
                    'size' => 20,
                    'required' => false
                );
            }
        }

        
        $fields_form[$i]['form']['submit'] = array(
            'title' => $this->l('Save settings'),
            'class' => 'btn btn-default pull-right'
        );


        $helper = new HelperForm();
         
        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
         
        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
         
        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = false;
        $helper->toolbar_scroll = false;
        $helper->submit_action = 'submit'.$this->name;
         
        // Load current value
        foreach ($this->params as $param_id => $param) {
            $helper->fields_value['SIRV_'.Tools::strtoupper($param_id)] =
            Configuration::get('SIRV_'.Tools::strtoupper($param_id));
        }

        $sirvjscomponent_items = explode(',', Configuration::get('SIRV_SIRVJSCOMPONENTS'));
        if (!empty($this->params['sirvjscomponents'])) {
            foreach (explode(',', $this->params['sirvjscomponents']['default']) as $item) {
                $helper->fields_value['SIRV_SIRVJSCOMPONENTS_'.$item] = in_array($item, $sirvjscomponent_items)?1:0;                
            }
        }

        return $helper->generateForm($fields_form);
    }

    public function getS3Client()
    {
        if ($this->s3client) {
            return $this->s3client;
        } else {
            require_once(dirname(__FILE__).'/sirv.api-s3.class.php');
            return $this->s3client = new SirvClient(
                Configuration::get('SIRV_S3_BUCKET'),
                Configuration::get('SIRV_S3_ACCESS_KEY'),
                Configuration::get('SIRV_S3_SECRET_KEY')
            );
        }
    }


    public function getAPIClient()
    {
        if ($this->APIClient) {
            return $this->APIClient;
        } else {
            require_once(dirname(__FILE__).'/sirv.api.class.php');
            return $this->APIClient = new SirvAPIClient(
                Configuration::get('SIRV_CLIENT_ID'),
                Configuration::get('SIRV_CLIENT_SECRET'),
                $this->defaultClient['clientId'],
                $this->defaultClient['clientSecret'],
                Configuration::get('SIRV_TOKEN'),
                Configuration::get('SIRV_TOKEN_EXPIRE_TIME'),
                'Sirv/PrestaShop '._PS_VERSION_.'/'.$this->version
            );
        }
    }

    public function fillDB()
    {
        $result = true;

        foreach ($this->getParams() as $param_id => $param) {
            $result = $result && (
                Configuration::get('SIRV_'.Tools::strtoupper($param_id))!='' ||
                Configuration::updateValue('SIRV_'.Tools::strtoupper($param_id), $param['default'])
            );
        }

        return $result;
    }

    public function getParams()
    {
        require_once('classes/SirvOptions.php');


        $translations = array(
            'Active' => $this->l('Active'),
            'Enable Sirv' => $this->l('Enable Sirv'),
            'Domain' => $this->l('Domain'),
            'Yes' => $this->l('Yes'),
            'No' => $this->l('No'),
            'Create and manage domains..' => $this->l('Create and manage domains in %syour Sirv account%s.'),
            'Image scaling' => $this->l('Image scaling'),
            'Add width/height' => $this->l('Add width/height'),
            
            'Add width/height attributes to img tags, if missing' =>
            $this->l('Add width/height attributes to img tags, if missing'),

            'Serve CMS images' => $this->l('Serve CMS images'),
            'Image profile' => $this->l('Image profile'),
            
            'Profiles can add watermarks, text overlays and other %seffects%s.' =>
            $this->l('Profiles can add watermarks, text overlays and other %seffects%s.'),
            
            'Create or change profiles on your %sSirv profiles page%s.' =>
            $this->l('Create or change profiles on your %sSirv profiles page%s.'),
            
            'Image quality' => $this->l('Image quality'),
            
            'Your Sirv default quality is specified in your' =>
            $this->l('Your Sirv default quality is specified in your'),
            
            'Serve perfectly scaled images to suit each users device, as recommended by Google PageSpeed Insights.' =>
            $this->l('Serve perfectly scaled images to suit each users device, as recommended by Google PageSpeed Insights.'),
            
            'Image optimisation' => $this->l('Image optimisation'),
            'Your Sirv default' => $this->l('Your Sirv default'),
            'Lazy loading' => $this->l('Lazy loading'),

            'Detect - add script only to pages that require it' => $this->l('Detect - add script only to pages that require it'),
            'All pages - always add script (select this if images are not loading)' => $this->l('All pages - always add script (select this if images are not loading)'),
            'No pages - don\'t add script (may break responsive images)' => $this->l('No pages - don\'t add script (may break responsive images)'),
            'Include Sirv JS' => $this->l('Include Sirv JS'),


            'Native lazy loading' => $this->l('Native lazy loading'),
            'Display background while image loads.' => $this->l('Display background until perfectly scaled image loads.'),
            'Sirv JS lazy loading' => $this->l('Sirv JS lazy loading'),
            'Disabled' => $this->l('Disabled'),
            'Lazy placeholder' => $this->l('Lazy placeholder'),
            

            'ImageLazy' => $this->l('Low quality image'),            
            'Grey background' => $this->l('Grey background'),
            'Blurred image' => $this->l('Blurred image'),
            'Disabled' => $this->l('Disabled'),
            'Exclude images' => $this->l('Exclude images'),

            'You can enter full URLs and the domain will be stripped' => 
            $this->l('You can enter full URLs and the domain will be stripped'),
            
            'Use * to specify all images at a certain path' => 
            $this->l('Use * to specify all images at a certain path'),

            'JS & CSS optimisation' => $this->l('JS & CSS optimisation'),
            'Delere old files' => $this->l('Delere old files'),
            'You can delete old files from Sirv which have been uploaded by the module more than 90 days and have mot been accessed more than 60 days.' => $this->l('You can delete old files from Sirv which were upload more than 90 days ago and not accessed for a 60 day period.'),
            'Serve JS, CSS and media files' => $this->l('Serve JS, CSS and media files'),
            
            'Speed-up your website by serving static files from Sirv\'s fast CDN (recommended).' =>
            $this->l('Speed-up your website by serving static files from Sirv\'s fast CDN (recommended).'),

            'Minify JS/CSS' => $this->l('Minify JS/CSS'),
            
            'Minify JS/CSS files to reduce size up to 80%.' =>
            $this->l('Minify JS/CSS files to reduce size up to 80%.'),

            'Occasionally, JS files can break, so test your site after enabling.' =>
            $this->l('Occasionally, JS files can break, so test your site after enabling.'),

            'Optimisation settings' => $this->l('Optimisation settings'),
            'Sirv JS features' => $this->l('Sirv JS features'),
            'Lazy & responsive images' => $this->l('Lazy & responsive images'),
            'Image zoom' => $this->l('Image zoom'),
            '360 spin' => $this->l('360 spin'),
            'Hotspots' => $this->l('Hotspots'),
            'Video streaming' => $this->l('Video streaming'),
            'Gallery (still images, YouTube & Vimeo videos)' => $this->l('Gallery (still images, YouTube & Vimeo videos)'),


            
            'Improve optimization by serving a smaller JS file, containing only the features you need.' =>
            $this->l('Improve optimization by serving a smaller JS file, containing only the features you need.'),


            'Sirv account' => $this->l('Sirv account'),
            'Bucket' => $this->l('Bucket'),
            'Sirv account' => $this->l('Sirv account'),
            'Access key' => $this->l('Access key'),
            'Secret key' => $this->l('Secret key'),
            'Network' => $this->l('Network'),
            'profile' => $this->l('profile'),
            'Folder name on Sirv' => $this->l('Folder name on Sirv'),

            'Images will be copied to this folder, autocreated in your Sirv account.' => $this->l( 'Images will be copied to this folder, autocreated in your Sirv account.'),
            'To use the same Sirv account on multiple PrestaShop sites' => $this->l('To use the same Sirv account on multiple PrestaShop sites'),
            'use a different folder name for each site.' => $this->l( 'use a different folder name for each site.'),
            'Sirv Media Viewer' => $this->l( 'Sirv Media Viewer'),
            'Extra content from Sirv' => $this->l( 'Extra content from Sirv'),
            'Add 360 spins, videos and images to your product gallery' => $this->l( 'Add 360 spins, videos and images to your product gallery'),
            'Left' => $this->l( 'Left'),
            'Right' => $this->l( 'Right'),
            'You can enter full URLs and the domain will be stripped' => $this->l( 'You can enter full URLs and the domain will be stripped'),

            'Extreme quality (huge filesize)' => $this->l('Extreme quality (huge filesize)'),
            'Very high quality' => $this->l('Very high quality'),
            'High quality (Sirv recommended default)' => $this->l('High quality (Sirv recommended default)'),
            'Medium quality' => $this->l('Medium quality'),
            'Low quality' => $this->l('Low quality'),
            'Very low quality' => $this->l('Very low quality'),
            

            'Pin a file with a particular filename' => $this->l('Pin a file with a particular filename'),
            'Enter characters to match either part or all of the filename e.g. dimensions.jpg or dimensions' => $this->l('Enter characters to match either part or all of the filename e.g. dimensions.jpg or dimensions'),

            


            'Layout' => $this->l('Gallery layout'),
            'Grid' => $this->l('Grid'),
            'Gallery' => $this->l('Slider'),
            'Spacing between items in the grid' => $this->l('Space between items'),
            'Aspect ratio of item(s)' => $this->l('Aspect ratio of items'),
            'Number of columns' => $this->l('Number of columns'),
            'Fullwidth featured image' => $this->l('Fullwidth featured image'),
            'Custom Aspect ratio' => $this->l('Custom aspect ratio'),
            
            '1 column' => $this->l('1 column'),
            '2 columns' => $this->l('2 columns'),
            '3 columns' => $this->l('3 columns'),
            '1 then 2' => $this->l('1 then 2'),
            '1 then 3' => $this->l('1 then 3'),

            'Miscellaneous' => $this->l('Miscellaneous'),
            'Custom CSS' => $this->l('Custom CSS'),
            'Custom CSS description' => $this->l('Add styles to fix any rendering conflicts caused by other CSS.'),

            'Enable Sirv Media Viewer' => $this->l('Enable Sirv Media Viewer'),
            'Content from Sirv' => $this->l('Content from Sirv'),
            'Folder structure on Sirv' => $this->l('Folder structure on Sirv'),
            'Possible variables' => $this->l('Possible variables'),
            'Folder structure for combinations' => $this->l('Folder structure for combinations'),
            'Cache TTL' => $this->l('Cache TTL'),
            'Time after which the cache entry expires' => $this->l('Time after which the cache entry expires'),
            'Disable cache (suitable for testing only)' => $this->l('Disable cache (suitable for testing only)'),
            'minute' => $this->l('minute'),
            'minutes' => $this->l('minutes'),
            'hour' => $this->l('hour'),
            'hours' => $this->l('hours'),
            'day' => $this->l('day'),
            'days' => $this->l('days'),
            'month' => $this->l('month'),
            'default setting, for typical use' => $this->l('default setting, for typical use'),
            'Order of content' => $this->l('Order of content'),
            'Content to show' => $this->l('Content to show'),
            'Show all images' => $this->l('Show all images'),
            'Show images for current combination only' => $this->l('Show images for current combination only'),
            'Sirv content first' => $this->l('Sirv content first'),
            'PrestaShop content first' => $this->l('PrestaShop content first'),
            'Only show Sirv content' => $this->l('Only show Sirv content'),

            '360 spin position' => $this->l('360 spin position'),
            'Video position' => $this->l('Video position'),
            'Pin video(s)' => $this->l('Pin video(s)'),
            'Pin spin(s)' => $this->l('Pin spin(s)'),
            'Pin images by file mask' => $this->l('Pin images by file mask'),
            'File mask' => $this->l('File mask'),

            'First' => $this->l('First'),
            'Second' => $this->l('Second'),
            'Third' => $this->l('Third'),
            'Last'     => $this->l('Last'),
            'Alphabetical' => $this->l('Alphabetical'),

            'Left' => $this->l('Left'),
            'Right' => $this->l('Right'),

            'Design and experience' => $this->l('Design and experience'),
            'Script options' => $this->l('Script options'),
            'CSS styling' => $this->l('CSS styling'),
            'Enable image zoom' => $this->l('Enable image zoom'),
            'Max height' => $this->l('Max height'),
            
            'Maximum height of viewer, including thumbnails' =>
            $this->l('Maximum height of viewer, including thumbnails'),
            
            'Synchronization settings' => $this->l('Synchronization settings'),
            'Delete images created by PrestaShop' => $this->l('Delete images created by PrestaShop'),
            'Delete' => $this->l('Delete'),
            'Keep' => $this->l('Keep'),

            'Delete images from Sirv server' => $this->l('Delete images from Sirv server'),
            'Automatically delete image from Sirv server when it deletes in Prestashop.' => $this->l('Automatically delete images from Sirv when they\'re deleted from PrestaShop.'),
            
            'Save disk space on your PrestaShop server by deleting resized images.' =>
            $this->l('Save disk space on your PrestaShop server by deleting resized images.'),
            
            '%sThumbnails can be regenerated%s if needed.' => $this->l('%sThumbnails can be regenerated%s if needed.'),
            'Exclude files/folders' => $this->l('Exclude files/folders'),
            'Use * to specify all files at a certain path' => $this->l('Use * to specify all files at a certain path'),
            
            'You can enter full URLs and the domain will be stripped' =>
            $this->l('You can enter full URLs and the domain will be stripped'),
            
            'Use * to specify all pages at a certain path' =>
            $this->l('Use * to specify all pages at a certain path'),

            'Username' => $this->l('Username'),
            'Password' => $this->l('Password'),
            'Exclude pages' => $this->l('Exclude pages'),



        );
        $options = new SirvOptions($this, $translations);
        return $options->getOptions();
    }

    public function isImageSynced($imgPathShort)
    {
        $sql = "SELECT COUNT(*) FROM "._DB_PREFIX_."sirv_images
        WHERE status = 'SYNCED' AND path = '".pSQL($imgPathShort)."'";
        $isSynced = Db::getInstance()->getValue($sql);
        return !empty($isSynced);
    }

    public function getSirvURL($full = true, $direct = false)
    {

        return (($full)?'https://':'').
        (
            (!empty(Configuration::get('SIRV_DOMAIN')))?Configuration::get('SIRV_DOMAIN'):
            (
                ($direct || empty(Configuration::get('SIRV_CDN_URL')))?
                Configuration::get('SIRV_S3_BUCKET').'.sirv.com':
                Configuration::get('SIRV_CDN_URL')
            )
        ).(($full)?'/':'');
    }

    private function checkIfImageExists($filename)
    {
        $this->APIClient = $this->getAPIClient();
        $stat = $this->APIClient->getFileStat($filename);
        return ($stat && !empty($stat->size));
    }

    private function checkIfURLExists($url)
    {

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_exec($ch);
        $valid = curl_getinfo($ch, CURLINFO_HTTP_CODE) == 200;
        curl_close($ch);
        return $valid;
    }

    public function uploadFile($imgPath, $sirvFileName, $imgURL = '', $data = array(), $wait = false)
    {

        $this->s3client = $this->getS3Client();
        $this->APIClient = $this->getAPIClient();

        if ((preg_match('/\/\/(localhost|127.0.0.1)/ims', _PS_BASE_URL_)
            || !empty(Configuration::get('SIRV_USE_S3')))) {
            return $this->APIClient->uploadFile($imgPath, $sirvFileName);
        } else {
            $GLOBALS['sirv_fetch_queue'][$imgURL] = array(
                'imgURL'        => $imgURL,
                'sirvFileName'  => $sirvFileName,
                'data'          => $data,
                'wait'          => $wait
            );

            return false;
        }
    }

    public function processFetchQueue()
    {
        
        if (empty($GLOBALS['sirv_fetch_queue'])) {
            return;
        }

        $username = Configuration::get('SIRV_HTTPUSERNAME');
        $password = Configuration::get('SIRV_HTTPPASSWORD');


        $images2fetch = array_chunk($GLOBALS['sirv_fetch_queue'], 20);
        foreach ($images2fetch as $images) {
            $imgs = $imgs_data = array();
            foreach ($images as $image) {
                $imgs_data[$image['sirvFileName']] = $image;

                $image['imgURL'] = $this->encodeURLParts($image['imgURL']);
                
                $imgs[] = array(
                    'url'       =>  $image['imgURL'],
                    'filename'  =>  preg_replace('/\/{1,}/ims', '/', $image['sirvFileName']),
                    'wait'      =>  !empty($image['wait'])?true:false
                );
                if (!empty($username) && !empty($password)) {
                    $imgs[count($imgs)-1]['auth'] = array(
                        'username'  =>  $username,
                        'password'  =>  $password,
                    );
                }

            }

            $res = $this->APIClient->fetchImage($imgs);

            if ($res) {
                if (!empty($res->result) && is_array($res->result)) {
                    foreach ($res->result as $result) {
                        $status = (!empty($result->success))?'SYNCED':'PROCESSING';
                        $msg = '';

                        if (empty($result->success) && isset($result->attempts) && is_array($result->attempts)) {
                            $attempt = end($result->attempts);
                            if (!empty($attempt->error) && !empty($attempt->error->httpCode)
                                && $attempt->error->httpCode == 429) {
                                preg_match(
                                    '/Retry after ([0-9]{4}\-[0-9]{2}\-[0-9]{2}.*?\([a-z]{1,}\))/ims',
                                    $attempt->error->message,
                                    $m
                                );
                                $time = strtotime($m[1]);
                                $this->APIClient->muteRequests($time);
                            }
                            $msg = $attempt->error->message;
                            $status = 'FAILED';
                        }

                        if (empty($result->success) && !is_array($result->attempts)) {
                            if (!empty($result->error) && 
                                preg_match(
                                    '/stop sending requests until ([0-9]{4}\-[0-9]{2}\-[0-9]{2}.*?\([a-z]{1,}\))/ims',
                                    $result->error,
                                    $m
                                )
                                ) {
                                $time = strtotime($m[1]);
                                $this->APIClient->muteRequests($time);
                            }
                            $msg = 'Rate limit exceeded';
                            $status = 'FAILED';
                        }

                        // if (preg_match('/Maximum file size exceeded/ims', $msg)) {
                        //     Db::getInstance()->delete(
                        //         'sirv_images',
                        //         'id = '.(int)$image['data']['id']
                        //     );
                        //     continue;                            
                        // }

                        $image = $imgs_data[$result->filename];
                        Db::getInstance()->update('sirv_images', array(
                            'size'              => (int)$image['data']['size'],
                            'sirvpath'          => pSQL($image['data']['imgSirvPath']),
                            'timestamp'         => pSQL(date('Y-m-d H:i:s', filemtime($image['data']['imgPath']))),
                            'timestamp_synced'  => pSQL(date('Y-m-d H:i:s', filemtime($image['data']['imgPath']))),
                            'msg'               => (empty($msg))?'NULL':pSQL($msg),
                            'status'            => pSQL($status),
                        ), 'id = '.(int)$image['data']['id']);

                        if (!empty($result->success)) {
                            if (Configuration::get('SIRV_DELETE_THUMBNAILS')=='Delete') {
                                if (!empty($image['data']['imageType'])) {
                                    foreach ($this->ImageTypes as $type_name => $type) {
                                        foreach(['', '2x'] as $x2) {
                                            $type = $type;
                                            $newFilePath = preg_replace(
                                                '/(.*)(\.[a-z]{1,})$/ims',
                                                '$1-'.$type_name.'$2',
                                                $image['data']['imgPath']
                                            );
                                            @unlink($newFilePath);
                                            $newFilePath = preg_replace(
                                                '/(.*)(\.[a-z]{1,})$/ims',
                                                '$1-'.$type_name.'2x'.'$2',
                                                $image['data']['imgPath']
                                            );
                                            @unlink($newFilePath);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        unset($GLOBALS['sirv_fetch_queue']);
    }

    public function deleteFile($file)
    {
        $this->APIClient = $this->getAPIClient();
        return $this->APIClient->deleteFile($file);
    }
 
    public function getImageTypes()
    {
        if (empty($this->ImageTypes)) {
            $this->ImageTypes = array();
            $sql = 'SELECT * FROM `'._DB_PREFIX_.'image_type` ORDER BY `id_image_type` ASC';
            $result = Db::getInstance()->ExecuteS($sql);
            foreach ($result as $row) {
                $this->ImageTypes[$row['name']] = $row;
            }
        }
        return $this->ImageTypes;
    }


    public function convertTypeToParams($type)
    {
        $profile = Configuration::get('SIRV_PROFILE');
        $quality = (int)Configuration::get('SIRV_QUALITY');

        if (empty($type)) {
            return '';
        } elseif (isset($this->ImageTypes[$type])) {
            return '?w='.$this->ImageTypes[$type]['width'].
            '&h='.$this->ImageTypes[$type]['height'].
            '&canvas.width='.$this->ImageTypes[$type]['width'].
            '&canvas.height='.$this->ImageTypes[$type]['height'].
            (!empty($profile)?'&profile='.urldecode($profile):'').
            (!empty($quality)?'&q='.$quality:'')
            ;
        }
    }

    private function getTotalNumberOfImages()
    {
        $n = 0;

        $sql = 'SELECT COUNT(*) FROM '._DB_PREFIX_.'image';
        $n += Db::getInstance()->getValue($sql);

        $tools = array('magic360', 'magicslideshow', 'magicscroll');
        foreach ($tools as $tool) {
            $sql_360 = 'SHOW TABLES LIKE "%'._DB_PREFIX_.$tool.'_images"';
            $row_360 = Db::getInstance()->ExecuteS($sql_360);
            if ($row_360) {
                $sql = 'SELECT COUNT(*) FROM '._DB_PREFIX_.$tool.'_images';
                $n += Db::getInstance()->getValue($sql);
            }
        }

        $sql = 'SELECT COUNT(*) FROM '._DB_PREFIX_.'sirv_images';
        $sirv_images_n = Db::getInstance()->getValue($sql);

        return max($n, $sirv_images_n);
    }

    public function getCacheInfo()
    {
        $stat = array(
                'NEW' => array('count'=>0,'size'=>0,'size_s'=>''),
                'PROCESSING' => array('count'=>0,'size'=>0,'size_s'=>''),
                'SYNCED' => array('count'=>0,'size'=>0,'size_s'=>''),
                'FAILED' => array('count'=>0,'size'=>0,'size_s'=>''),
                'q' => 0,
                'size_s' => '-',
                'size' => 0,
                'total_by_type' => array(
                    'PRODUCT'=>0,
                    'CATEGORY'=>0,
                    'CUSTOM'=>0,
                    'MAGIC360'=>0,
                    'MAGICSCROLL'=>0,
                    'MAGICSLIDESHOW'=>0,
                ),
                'total_count' => (int)$this->getTotalNumberOfImages(),
        );

        $sql = 'SELECT status, count(*) as `count`, SUM(size) as size FROM '._DB_PREFIX_.'sirv_images GROUP BY status';
        $results = Db::getInstance()->ExecuteS($sql);
        if ($results) {
            foreach ($results as $row) {
                $stat[$row['status']] = array(
                    'count'=>(int)$row['count'],
                    'size'=>(int)$row['size'],
                    'size_s'=>$this->getFormatedFileSize((int)$row['size']),
                );
            }

            $stat['size_s'] = $stat['SYNCED']['size_s'];
            $stat['size'] = (int)$stat['SYNCED']['size'];
            $stat['q'] = (int)$stat['SYNCED']['count'];

            $sql = 'SELECT `type`,count(*) as `count` FROM '
            ._DB_PREFIX_.'sirv_images GROUP BY `type` ORDER BY count DESC';
            $results = Db::getInstance()->ExecuteS($sql);
            if ($results) {
                foreach ($results as $row) {
                    $stat['total_by_type'][$row['type']] = $row['count'];
                }
            }

            $stat['total_by_type']['MAGIC'] = (int)$stat['total_by_type']['MAGIC360']+
            (int)$stat['total_by_type']['MAGICSLIDESHOW']+
            (int)$stat['total_by_type']['MAGICSCROLL'];

            return $stat;
        }
        return $stat;
    }


    public static function getFormatedFileSize($bytes, $fileName = "", $decimal = 2, $bytesInMM = 1000)
    {
        if (!empty($fileName)) {
            $bytes = filesize($fileName);
        }

        $sign = ($bytes>=0)?'':'-';
        $bytes = abs($bytes);

        if (is_numeric($bytes)) {
            $position = 0;
            $units = array( " Bytes", " KB", " MB", " GB", " TB" );
            while ($bytes >= $bytesInMM && ($bytes / $bytesInMM) >= 1) {
                 $bytes /= $bytesInMM;
                 $position++;
            }
            return ($bytes==0)?'':$sign.round($bytes, $decimal).$units[$position];
        } else {
            return "";
        }
    }

    private function getFirstLastName($name)
    {
        $name = trim($name);
        $last_name = (strpos($name, ' ') === false) ? '' : preg_replace('#.*\s([\w-]*)$#', '$1', $name);
        $first_name = trim(preg_replace('#'.$last_name.'#', '', $name, 1));
        return array($first_name, $last_name);
    }

    public function ajaxProcessSirvInitAccount()
    {
        $this->APIClient = $this->getAPIClient();

        if (empty($_REQUEST['existing_account']) && !empty($_REQUEST['createAccount'])) {

            $_REQUEST['sirv_password'] = trim($_REQUEST['sirv_password']);

            if (empty($_REQUEST['sirv_lastName'])) {
                $_REQUEST['sirv_lastName'] = $_REQUEST['sirv_firstName'];
            }

            list($_REQUEST['sirv_firstName'], $_REQUEST['sirv_lastName']) =
            $this->getFirstLastName($_REQUEST['sirv_firstName']);

            if ($_REQUEST['sirv_password'] == '') {
                echo json_encode(
                    array('error' => $this->l('Password is empty.'))
                );
                return;
            }

            if (!preg_match('/^.{8,127}$/ims', $_REQUEST['sirv_password'])) {
                echo json_encode(
                    array(
                        'error' =>
                        $this->l('Choose a password at least 8 characters long and less than 128 characters.')
                    )
                );
                return;
            }

            if ($_REQUEST['sirv_firstName'] == '') {
                echo json_encode(
                    array('error' => $this->l('First name is empty.'))
                );
                return;
            }
            if (!preg_match('/^.{2,35}$/ims', $_REQUEST['sirv_firstName'])) {
                echo json_encode(
                    array('error' => $this->l('First name must be 2-35 characters.'))
                );
                return;
            }

            if ($_REQUEST['sirv_lastName'] == '') {
                echo json_encode(
                    array('error' => $this->l('Last name is empty.'))
                );
                return;
            }
            if (!preg_match('/^.{2,35}$/ims', $_REQUEST['sirv_lastName'])) {
                echo json_encode(
                    array('error' => $this->l('Last name must be 2-35 characters.'))
                );
                return;
            }

            $isEmail  = preg_match(
                '/^[_a-z0-9-\+]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$/',
                $_REQUEST['sirv_email']
            );

            if (!$isEmail) {
                echo json_encode(
                    array('error' => $this->l('Please enter correct email.'))
                );
                return;
            }

            if (preg_match('/\@mail\.ru/ims', $_REQUEST['sirv_email'])) {
                echo json_encode(
                    array('error' => $this->l('Please use a company email address (not mail.ru).'))
                );
                return;
            }


            $_REQUEST['sirv_alias'] = trim($_REQUEST['sirv_alias']);

            if (!preg_match('/^[a-z0-9\-]{6,30}$/ims', $_REQUEST['sirv_alias'])) {
                echo json_encode(
                    array('error' => $this->l('Wrong account name.').' '.
                        $this->l('There should be 6-30 characters (letters, numbers and hyphens).')
                    )
                );
                return;
            }

            if (preg_match('/^.*\-((srv[0-9]{1,})|direct|cdn)$/ims', $_REQUEST['sirv_alias'])) {
                echo json_encode(
                    array('error' => 'Account name '.$_REQUEST['sirv_alias'].' is not permitted.'.' '.
                        'You could try '.
                        preg_replace('/^(.*)\-((srv[0-9]{1,})|direct|cdn)$/ims', '$1$2', $_REQUEST['sirv_alias']).
                        ' instead.'
                    )
                );
                return;
            }

            $account = $this->APIClient->registerAccount(
                trim(Tools::strtolower($_REQUEST['sirv_email'])),
                trim(Tools::strtolower($_REQUEST['sirv_password'])),
                trim(Tools::strtolower($_REQUEST['sirv_firstName'])),
                trim(Tools::strtolower($_REQUEST['sirv_lastName'])),
                trim(Tools::strtolower($_REQUEST['sirv_alias']))
            );
            if (!$account) {
                $lastResp = $this->APIClient->getLastResponse();
                if ($lastResp->result->message == 'Supplied data is not valid' &&
                    !empty($lastResp->result->validationErrors) &&
                    preg_match('/AccountAlias/ims', $lastResp->result->validationErrors[0]->message)) {
                    $lastResp->result->message = $this->l('Wrong value for account name. Please fix it.');
                }
                
                if ($lastResp->result->message == 'Duplicate entry') {
                    $lastResp->result->message =
                    $this->l('That email address is already registered. Please login instead.');
                }

                echo json_encode(
                    array('error' => preg_replace(
                        '/child.*?fails because \[(.*?)\]/ims',
                        '$1',
                        $lastResp->result->message
                    ))
                );
                return;
            }
        }

        if (empty($_REQUEST['sirv_account']) &&
            !empty($_REQUEST['sirv_email']) &&
            !empty($_REQUEST['sirv_password'])
        ) {
            //$users = $this->APIClient->getUsersList($_REQUEST['sirv_email'], $_REQUEST['sirv_password']);

            if (empty($_REQUEST['sirv_otp'])) {
                $_REQUEST['sirv_otp'] = '';
            }

            $users = $this->APIClient->getUsersList($_REQUEST['sirv_email'], $_REQUEST['sirv_password'], $_REQUEST['sirv_otp']);
            if ($users == 'OTPREQUIRES') {
                echo json_encode( ['otprequires' => 1] );
                return;
            }

            if (empty($users) || !is_array($users)) {
                $lastResp = $this->APIClient->getLastResponse();

                $lastResp_msg = (isset($lastResp->result->message))?$lastResp->result->message:'';

                $message = $this->l('An error occurred.');
                if ($lastResp_msg == 'Forbidden') {
                    if (!empty($_REQUEST['sirv_otp'])) {
                        $message = $this->l('Your authentication code was incorrect. Please try again.');
                    } else {
                        $message = 'SIRV_MSG1';
                    }
                } else {
                    $message = 'SIRV_MSG2';
                }
                echo json_encode(
                    array('error' => $message)
                );
                return;
            } else {
                echo json_encode(
                    array('users' => $users)
                );
                return;
            }



        } elseif (!empty($_REQUEST['sirv_account'])) {
            $res = $this->APIClient->setupClientCredentials($_REQUEST['sirv_account']);
            Configuration::updateValue('SIRV_ACCOUNT_EMAIL', trim(Tools::strtolower($_REQUEST['sirv_email'])));
            if ($res) {
                $res = $this->APIClient->setupS3Credentials(Configuration::get('SIRV_ACCOUNT_EMAIL'));
                if ($res) {
                    $this->APIClient = $this->getAPIClient();
                    if (!$this->checkIfImageExists(Configuration::get('SIRV_FOLDER').'/1x1.png')) {
                        $this->APIClient->uploadFile(
                            dirname(__FILE__).'/views/img/1x1.png',
                            '/'.Configuration::get('SIRV_FOLDER').'/1x1.png'
                        );
                    }

                    echo json_encode(
                        array('connected' => '1')
                    );
                    return;
                }
            }
            echo json_encode(
                array('error' => $this->l('An error occurred.'))
            );
            return;
        }

        echo json_encode(
            array('error' => $this->l('An error occurred.'))
        );

        die();
    }

    public function isMuted()
    {
        return ((int)Configuration::get('SIRV_MUTE') > time());
    }

    public function encodeURLParts($url)
    {
        preg_match('/^(https?:\/\/.*?\/)/ims', $url, $m);
        $url = preg_replace('/^(https?:\/\/.*?\/)/ims', '', $url);
        $parts = explode('/', $url);
        foreach ($parts as $pi => $pip) {
            $parts[$pi] = rawurlencode($pip);
        }
        return $m[1].implode('/', $parts);
    }

    public function decodeURLParts($url)
    {
        preg_match('/^(https?:\/\/.*?\/)/ims', $url, $m);
        $url = preg_replace('/^(https?:\/\/.*?\/)/ims', '', $url);
        $parts = explode('/', $url);
        foreach ($parts as $pi => $pip) {
            $parts[$pi] = rawurldecode($pip);
        }
        return $m[1].implode('/', $parts);
    }


    // product videos

    public function loadProductVideoData($id_product)
    {
        $data = Db::getInstance()->getValue(
            'SELECT data FROM `'._DB_PREFIX_.'sirv_video` WHERE id_product='.(int)$id_product
        );
        if (empty($data)) {
            return array();
        }
        return unserialize($data);
    }

    public function saveProductVideoData($id_product, $value)
    {
        $value = $this->prepareProductVideosDataForSave($value);
        $value = serialize($value);
        $data = Db::getInstance()->ExecuteS(
            'SELECT * FROM `'._DB_PREFIX_.'sirv_video` WHERE id_product='.(int)$id_product
        );
        if (empty($data)) {
            $sql = 'INSERT INTO `'.
            _DB_PREFIX_.'sirv_video` (`id_product`, `data`) VALUES ('.(int)$id_product.', \''.pSQL($value).'\')';
        } else {
            $sql = 'UPDATE `'.
            _DB_PREFIX_.'sirv_video` SET `data`=\''.pSQL($value).'\' WHERE `id_product`='.(int)$id_product;
        }
        Db::getInstance()->Execute($sql);
    }

    public function prepareProductVideosDataForSave($value)
    {

        $data = array();

        if (empty($value)) {
            return $data;
        }

        $urls = preg_split('#\n++|\s++#', $value, -1, PREG_SPLIT_NO_EMPTY);

        foreach ($urls as $_url) {
            $url = parse_url($_url);
            if (!$url) {
                $data[$_url] = array();
                continue;
            }

            $isVimeo = false;
            $videoCode = null;
            if (preg_match('#youtube\.com|youtu\.be#', $url['host'])) {
                if (isset($url['query']) && preg_match('#\bv=([^&]+)(?:&|$)#', $url['query'], $matches)) {
                    $videoCode = $matches[1];
                } elseif (isset($url['path'])
                    && preg_match('#^/(?:embed/|v/)?([^/\?]+)(?:/|\?|$)#', $url['path'], $matches)) {
                    $videoCode = $matches[1];
                }
            } elseif (preg_match('#(?:www\.|player\.)?vimeo\.com#', $url['host'])) {
                $isVimeo = true;
                if (isset($url['path'])
                    && preg_match(
                        '#/(?:channels/[^/]+/|groups/[^/]+/videos/|album/[^/]+/video/|video/|)(\d+)(?:/|\?|$)#',
                        $url['path'],
                        $matches
                    )) {
                    $videoCode = $matches[1];
                }
            }

            if (!$videoCode) {
                $data[$_url] = array();
                continue;
            }

            if ($isVimeo) {
                $hash = unserialize(Tools::file_get_contents('http://vimeo.com/api/v2/video/'.$videoCode.'.php'));
                $thumb = $hash[0]['thumbnail_small'];
            } else {
                $thumb = 'https://i1.ytimg.com/vi/'.$videoCode.'/1.jpg';
            }

            $data[$_url] = array(
                'code' => $videoCode,
                'thumb' => $thumb,
                'vimeo' => $isVimeo,
                'youtube' => !$isVimeo,
            );
        }

        return $data;
    }


    public function loadProductAssetsData($id_product)
    {
        $data = Db::getInstance()->getValue(
            'SELECT data FROM `'._DB_PREFIX_.'sirv_assets` WHERE id_product='.(int)$id_product
        );
        if (empty($data)) {
            return array();
        }
        return unserialize($data);
    }

    public function saveProductAssetsData($id_product, $value)
    {

        if (!isset($_REQUEST['sirv_assets'])) return;

        $value = $this->prepareProductAssetsDataForSave($value);
        $value = serialize($value);
        $data = Db::getInstance()->ExecuteS(
            'SELECT * FROM `'._DB_PREFIX_.'sirv_assets` WHERE id_product='.(int)$id_product
        );
        if (empty($data)) {
            $sql = 'INSERT INTO `'.
            _DB_PREFIX_.'sirv_assets` (`id_product`, `data`) VALUES ('.(int)$id_product.', \''.pSQL($value).'\')';
        } else {
            $sql = 'UPDATE `'.
            _DB_PREFIX_.'sirv_assets` SET `data`=\''.pSQL($value).'\' WHERE `id_product`='.(int)$id_product;
        }
        Db::getInstance()->Execute($sql);
    }


    public function prepareProductAssetsDataForSave($value)
    {

        $data = array();

        if (empty($value)) {
            return $data;
        }

        $urls = preg_split('#\n++|\s++#', $value, -1, PREG_SPLIT_NO_EMPTY);

        foreach ($urls as $_url) {
            $url = parse_url($_url);
            if (!$url) {
                $data[$_url] = array();
                continue;
            }

            $data[$_url] = array(
                $url
            );
        }

        return $data;
    }

    public function hookDisplayAdminProductsExtra($params)
    {

        if ($this->isPrestaShop17x) {
            $id_product = $params['id_product'];
        } else {
            $id_product = Tools::getValue('id_product');
        }
        $this->context->smarty->assign(array(
            'legacy_template' => false,
            'p16x_template' => $this->isPrestaShop16x,
        ));
        $data = $this->loadProductVideoData($id_product);
        if (!empty($data)) {
            $urls = array_keys($data);
            $urls = implode("\n", $urls)."\n";
            $invalid_urls = array();
            foreach ($data as $url => $_data) {
                if (empty($_data)) {
                    $invalid_urls[] = $url;
                }
            }
            $this->context->smarty->assign(array(
                'sirv_textarea' => $urls,
                'sirv_invalid_urls' => empty($invalid_urls) ? null : $invalid_urls
            ));
        }
        $data = $this->loadProductAssetsData($id_product);
        if (!empty($data)) {
            $urls = array_keys($data);
            $urls = implode("\n", $urls)."\n";
            $invalid_urls = array();
            foreach ($data as $url => $_data) {
                if (empty($_data)) {
                    $invalid_urls[] = $url;
                }
            }
            $this->context->smarty->assign(array(
                'sirv_textarea_assets' => $urls,
                'sirv_invalid_urls_assets' => empty($invalid_urls) ? null : $invalid_urls
            ));
        }
        if ($this->isPrestaShop17x) {
            return $this->display(__FILE__, 'views/templates/admin/product_videos_ps17.tpl');
        }
        return $this->display(__FILE__, 'views/templates/admin/product_videos.tpl');
    }

    public function hookAfterSaveProduct($params)
    {
        $id_product = Tools::getValue('id_product');
        $productVideos = Tools::getValue('sirv_video');
        $productAssets = Tools::getValue('sirv_assets');
        $this->saveProductVideoData($id_product, $productVideos);
        $this->saveProductAssetsData($id_product, $productAssets);
        //$this->updateProductImages($id_product);
    }

    public function hookActionProductUpdate($params)
    {
        $id_product = Tools::getValue('id_product');
        $productVideos = Tools::getValue('sirv_video');
        $productAssets = Tools::getValue('sirv_assets');
        $this->saveProductVideoData($id_product, $productVideos);
        $this->saveProductAssetsData($id_product, $productAssets);
        //$this->updateProductImages($id_product);
    }

    public function hookActionProductSave($params)
    {
        $id_product = Tools::getValue('id_product');
        $productVideos = Tools::getValue('sirv_video');
        $productAssets = Tools::getValue('sirv_assets');
        $this->saveProductVideoData($id_product, $productVideos);
        $this->saveProductAssetsData($id_product, $productAssets);
        //$this->updateProductImages($id_product);
    }

    public function isFileExcluded($path, $option = 'SIRV_EXCLUDEFILES')
    {
        $excludedFiles = Configuration::get($option);

        $excludedFiles = str_replace(
            str_replace('http:', 'https:', _PS_BASE_URL_.__PS_BASE_URI__),
            '',
            str_replace('http:', 'https:', $excludedFiles)
        );

        $excludedFiles = explode("\n", $excludedFiles);

        if (empty($excludedFiles)) {
            return false;
        }

        foreach ($excludedFiles as $i => &$pattern) {
            $pattern = trim($pattern);
            if (empty($pattern)) {
                unset($excludedFiles[$i]);
            } else {
                $pattern = '.*'.str_replace(
                    '*',
                    '.*',
                    str_replace('\*', '*', preg_quote(
                        $pattern,
                        '#'
                    ))
                ).'.*';
            }
        }
        $regExp = implode('|', $excludedFiles);

        return !empty($regExp) && @preg_match('#' . $regExp . '#ims', $path);
    }

    public function isPageExcluded()
    {
        return $this->isFileExcluded($_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'], 'SIRV_EXCLUDEPAGES');
    }

    public function getRemoteURLSize($url, $compressed = false)
    {

        $ch = curl_init($url);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        if ($compressed) {
            curl_setopt($ch, CURLOPT_ENCODING, '');
        }
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);
        curl_exec($ch);

        $data = curl_exec($ch);
        $size = (int)curl_getinfo($ch, CURLINFO_SIZE_DOWNLOAD);

        curl_close($ch);

        return $size;
    }

    public function ajaxProcessSirvGetSirvJsSize()
    {

        $modules = Tools::getValue('modules');
        if (empty($modules)) $modules = [];
        $modules = implode(',', $modules);

        if ($modules == '') {
            echo json_encode(
                array(
                    'size' => $this->l('Sirv JS won\'t be loaded')
                )
            );            
        } else {
            $url = 'https://scripts.sirv.com/sirvjs/v3/sirv.js?modules='.$modules;
            echo json_encode(
                array(
                    'size' => $this->getFormatedFileSize($this->getRemoteURLSize($url, true)).' '.
                    '('.$this->l('unzipped').' '.$this->getFormatedFileSize($this->getRemoteURLSize($url, false)).')'
                )
            );            
        }

        die();
    }

}
